# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""
This module implements functions that provide automatic word completion for the
text editor.

Automatic word completion works by scanning the document for strings that
begin with the first four or more characters after the cursor. The characters
must be alphanumeric.

When matches are found, a completion window appears displaying a list of the
found matches. Users can then use either the up and down arrow keys or the
mouse pointer to select and activate the desired string for word completion.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""

from re import findall, UNICODE

from gtk.gdk import WINDOW_TYPE_HINT_MENU
from gtk import TreeView, ListStore, CellRendererText, TREE_VIEW_COLUMN_AUTOSIZE
from gtk import TreeViewColumn, TEXT_WINDOW_WIDGET, Window, WINDOW_POPUP
from gtk import POLICY_AUTOMATIC, ScrolledWindow, SHADOW_IN

from cursor import get_string_to_cursor, get_cursor_iter
from utility import update_status_info
from internationalization import msg0155


def create_completion_window():
	u"""
	Create the completion window.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	@return: A completion window.
	@rtype: A gtk.Window.

	"""

	window = Window(WINDOW_POPUP)

	# Set wigdet properties.

	window.set_property("height-request", 200)
	window.set_property("width-request", 200)

	# Set window properties.

	window.set_property("accept-focus", True)
	window.set_property("can-focus", True)
	window.set_property("allow-grow", False)
	window.set_property("allow-shrink", False)
	window.set_property("decorated", False)
	window.set_property("resizable", False)
	window.set_property("skip-pager-hint", True)
	window.set_property("skip-taskbar-hint", True)
	window.set_property("type-hint", WINDOW_TYPE_HINT_MENU)
	window.set_property("urgency-hint", False)

	return window


def hide_completion_window(editor):
	u"""
	Hide the completion window.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	try:

		editor.completion_window.hide_all()
		editor.view.destroy()

		# The completion flag indicates whether or not the completion window is
		# visible. If the flag is true, the completion window is visible.
		# Otherwise, it is not.

		editor.completion_flag = False

	except AttributeError:

		pass

	return


def create_completion_view(match_list):
	u"""
	Create the treeview to house the completion window's list of potential
	completion strings

	@param match_list: A list containing potential completion strings.
	@type match_list: A list object.

	"""

	# Create a treeview to house potential word completion strings.

	completion_view = TreeView()

	# Create a liststore to serve as a model for completion strings.

	model = ListStore(str)

	# Populate the model with completion matches.

	for value in match_list:

		model.append([value])

	# Create a renderer for the completion strings.

	renderer = CellRendererText()

	# Create a column for the completion strings.

	column = TreeViewColumn(u"", renderer, text=0)
	completion_view.append_column(column)
	column.set_expand(True)
	column.set_sizing(TREE_VIEW_COLUMN_AUTOSIZE)

	# Set the treeview properties.

	completion_view.set_model(model)
	completion_view.columns_autosize()
	completion_view.set_headers_visible(False)
	completion_view.set_property("rules-hint", True)
	completion_view.set_property("hover-selection", True)

	return completion_view


def get_position(editor):
	u"""
	Get the position to show the completion window.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	@return: Return the position where the completion window will be placed.
	@rtype: An integer object representing pixels.

	"""

	# Get the cursor's iterator.

	cursor_iter = get_cursor_iter(editor)

	# Get the buffer cordinates via the cursor's iterator.

	rectangle = editor.text_view.get_iter_location(cursor_iter)

	# Get the window cordinates via the buffer cordinates.

	win_x, win_y = editor.text_view.buffer_to_window_coords(TEXT_WINDOW_WIDGET,
												rectangle.x,
												rectangle.y+rectangle.height)

	# Get the gtk.gdk.Window for the editor's textview.

	window = editor.text_view.get_window(TEXT_WINDOW_WIDGET)

	# Get the editor's textview cordinates.

	window_x, window_y = window.get_origin()

	return window_x+win_x, window_y+win_y


def update_completion_view(editor, match_list):
	u"""
	Update the completion window list.

	@param editor: Reference to the editor object.
	@type editor: An editor object.

	@param match_list: A list of potential completion strings.
	@type match_list: A list object.

	"""

	# Create a liststore to serve as a model for completion strings.

	model = ListStore(str)

	# Populate the model if there is a match. Hide the completion_window if
	# there are no matches.

	if match_list:

		for value in match_list:

			model.append([value])

		editor.view.set_model(model)
		column = editor.view.get_column(0)
		editor.view.set_cursor((0,), column, start_editing=False)
		size = editor.view.size_request()
		editor.completion_window.set_property("height-request", size[1]+2)

	else:

		hide_completion_window(editor)

	return


def insert_word_completion(path, editor):
	u"""
	Insert items selected in the completion window into the text editor's
	buffer.

	@param path: The selected row in the completion window.
	@type path: A gtk.TreeRow object.

	@param editor: Reference to the editor object.
	@type editor: An editor object.

	"""

	# Get the database containing potential completion string matches.

	model = editor.view.get_model()

	# Get the selected completion string.

	completion_string = model[path[0]][0].decode("utf8")

	# Index to split completion string for insertion into the text editor's
	# buffer. Encode to utf8 before insertion.

	index = len(editor.valid_string.decode("utf8"))
	string = completion_string[index:]

	# Split completion_string at the right index and insert into the editor's
	# buffer.

	editor.text_buffer.insert_at_cursor(string)

	# Feedback to the status bar indicating word completion occurred.

	update_status_info(editor, msg0155, "yes")

	return


def row_activated_cb(treeview, path, column, editor):
	u"""
	Handles callback when item in completion window is selected and activated.

	@param path: The selected row in the completion window.
	@type path: A gtk.TreeRow object.

	@param column: The column holding the completion list.
	@type column: A gtk.TreeViewColumn

	@param editor: Reference to an editor object.
	@type editor: An editor object.

	"""

	# Insert selected word into the text editor's buffer and hide there
	# completion window.

	insert_word_completion(path, editor)
	hide_completion_window(editor)

	return


def button_press_event_cb(view, event):
	u"""
	Handles callback when an item in the completion	window is clicked on.

	@param view: The completion window's treeview.
	@type view: A gtk.TreeView object.

	@param event: An event generated by clicking on the mouse button.
	@type event: A gtk.Event object.

	"""

	# Get the selected item on the completion window.

	selection = view.get_selection()

	# Get the model and iterator of the selected item.

	model, iterator = selection.get_selected()

	# Get the path and column of the selected item.

	path = model.get_path(iterator)
	column = view.get_column(0)

	# Activate the selected item to trigger word completion.

	view.row_activated(path, column)

	return


def show_completion_window(editor, match_list):
	u"""
	Show the completion window containing potential word completion strings.

	@param editor: Reference to the editor object.
	@type editor: An editor object.

	@param match_list: A list of potential completion strings.
	@type match_list: A list object.

	@return: A popup winow showing found matches
	@rtype: A gtk.Window.

	"""

	# Create a window to contain completion_view.

	completion_window = create_completion_window()

	# Position to place the the completion_window.

	position_x, position_y = get_position(editor)

	# Set the completion window properties.

	completion_window.move(position_x, position_y)

	# Create a scrollwindow for the completion window.

	scrollwin = ScrolledWindow()
	scrollwin.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
	scrollwin.set_shadow_type(SHADOW_IN)
	completion_window.add(scrollwin)

	# Create the completion view to house completion matches.

	editor.view = create_completion_view(match_list)
	editor.view.connect("row-activated", row_activated_cb, editor)
	editor.view.connect("button-press-event", button_press_event_cb)
	scrollwin.add(editor.view)

	# Get the size of the treeview.

	width, height = editor.view.size_request()

	# Expand the witdh of the completion window dynamically.

	if width > 200:

		completion_window.set_property("width-request", width+20)

	# Constrain the height of completion window to 200 pixels.

	if height < 200:

		completion_window.set_property("height-request", height+2)

	if height > 200:

		completion_window.set_property("height-request", 215)

	# Show the completion_window.

	completion_window.show_all()

	# A flag indicating the visibility of the completion window. True if the
	# window is visible. False otherwise.

	editor.completion_flag = True

	return completion_window


def scan_buffer(editor, cursor_string):
	u"""
	Scan the text editor's buffer for potential matches for word completion.

	@param editor: Reference to the editor object.
	@type editor: An editor object.

	@param cursor_string: A string 3 characters long before the cursor
	@type cursor_string: A string.

	@return: A list containing potential matches for word completion or nothing.
	@rtype: A list object or None

	"""

	# Get iterator at the beginning and end of the editor's buffer.

	start, end = editor.text_buffer.get_bounds()

	# Get the contents of the editor's buffer.

	text = editor.text_buffer.get_text(start, end, include_hidden_chars=False)

	string = text.decode("utf8")

	# Match any character that starts with cursor_string and has one or more
	# alphanumeric characters.

	match = findall(r"\b"+cursor_string+r"[-*\w+\b]+", string, UNICODE)

	# If matches are found, filter the found matches to remove duplicate
	# entries, if any, and then return a list of found matches.

	if match:

		filtered_match = []

		for value in match:

			if value in filtered_match:

				continue

			else:

				filtered_match.append(value)

		del match

		# Throw an exception if a completion window has never been created.
		# In the process, create a new completion window. If a completion window
		# has been created, check the flags to see whether or not the completion
		# window is visible.
		#
		# The completion_flag indicates the visibility of the completion window.
		# If the flag is true, the completion window is visible, otherwise it
		# is not.
		#
		# If the completion window is visible, update the treeview depending on
		# input from user. It it isn't, update the treeview and then show then
		# the completion window. Note, the completion window is never destroyed.
		# It is only hidden or shown.

		if filtered_match:

			try:

				if editor.completion_flag:

					update_completion_view(editor, filtered_match)

				else:

					editor.completion_window = show_completion_window(editor,
													filtered_match)

			except AttributeError:

				editor.completion_window = show_completion_window(editor,
													filtered_match)
		else:

			hide_completion_window(editor)

	else:

		hide_completion_window(editor)

	return


def validate_cursor_string(editor):
	u"""
	Check if the characters before the cursor meet the criteria for automatic
	word completion.

	Automatic word completion is activated only after three alphanumeric
	characters	can be retrieved before the cursor.

	@param editor: Reference to the editor object.
	@type editor: An editor object.

	"""

	# Get the string or word before the cursor

	valid_string = get_string_to_cursor(editor)

	# If the string meets the criteria for automatic completion, scan the buffer
	# for possible completion strings.

	if valid_string:

		valid_string.decode("utf8")
		editor.valid_string = valid_string
		scan_buffer(editor, valid_string)

	else:

		# Hide the completion window if a valid string for automatic completion
		# can not be found.

		hide_completion_window(editor)

	return
