# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""
The module contains functions that define all bracket operations of the text
editor. They include finding matching brackets, removing matching brackets,
completing matching brackets to mention a few.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""

from gtk.gdk import beep

try:

	from gtksourceview import source_iter_find_matching_bracket as match

except ImportError:

	from gtksourceview import gtk_source_iter_find_matching_bracket as match

from utility import update_status_info
from internationalization import msg0020, msg0021, msg0154, msg0156
from cursor import get_cursor_iter


def match_bracket(editor):
	u"""
	Find matching brackets

	@param editor: an editor instance
	@type editor: an editor object
	"""

	# We use this flag to fool and prevent the editor from switching to edit
	# mode when the user types into the buffer.

	editor.mode_edit_flag = True

	# Get cursor position

	cursor_mark = editor.text_buffer.get_insert()
	cursor_iter = editor.text_buffer.get_iter_at_mark(cursor_mark)

	# Find matching bracket.

	if match(cursor_iter):

		# msg0020 is a string eligible for internationalization and localization.
		# See the module internationalization.py for more information.

		update_status_info(editor, msg0020, u"yes")
		cursor_iter.forward_char()
		editor.text_buffer.place_cursor(cursor_iter)
		cursor_mark = editor.text_buffer.get_insert()
		editor.text_view.scroll_to_mark(cursor_mark, 0.3,
								 use_align=False, xalign=0.5,
								 yalign=0.5)

	else:

		# msg0021 is a string eligible for internationalization and localization.
		# See the module internationalization.py for more information.

		update_status_info(editor, msg0021, u"no")
		beep()

	return


def insert_completion(editor, character):
	u"""
	Inserts opposite bracket characters into the editor's buffer.

	@param editor: A reference to the editor object.
	@type editor: An editor object.

	@param character: ], ), }, or "
	@type character: A string.

	"""

	# Begin recording for undo operation.

	editor.text_buffer.begin_user_action()

	# We use this flag to fool and prevent the editor from switching to edit
	# mode when the user types into the buffer.

	editor.mode_edit_flag = True

	# Insert character into the editor's buffer.

	editor.text_buffer.insert_at_cursor(character)

	# Move the cursor between the brackets.

	cursor_mark = editor.text_buffer.get_insert()
	cursor_iter = editor.text_buffer.get_iter_at_mark(cursor_mark)
	cursor_iter.backward_char()
	editor.text_buffer.place_cursor(cursor_iter)

	editor.text_buffer.end_user_action()

	return


def activate_bracket_completion(editor, value):
	u"""
	Provide completion for {, [, " and ( characters.

	@param editor: A reference to the editor object.
	@type editor: An editor object.

	@param value: An integer value representing either {, [, " or (.
	@type value: An integer.

	"""

	if value == 123:

		insert_completion(editor, "{}")

	elif value == 91:

		insert_completion(editor, "[]")

	elif value == 34:

		insert_completion(editor, '""')

	else:

		insert_completion(editor, "()")

	# If the flag is true, when a backspace character is entered in the text
	# editor's buffer, matching brackets are removed.

	editor.backspace_flag = True

	# Feedback to the statusbar indicating a bracket completion operation
	# occurred.

	update_status_info(editor, msg0154, u"yes")

	return


def remove_matching_brackets(editor):
	u"""
	Remove matching brackets in the text editor's buffer.

	Sometimes users accidentally type brackets into the text editor's buffer.
	On such occassions pressing the backspace key activates this function to
	remove the matching brackets.

	@param editor: Reference to the editor instance.
	@type editor: An editor object.

	"""

	# Begin recording for undo operation.

	editor.text_buffer.begin_user_action()

	# We use this flag to fool and prevent the editor from switching to edit
	# mode when the user types into the buffer.

	editor.mode_edit_flag = True

	# Move iterators before and after matching brackets.

	cursor_iter = get_cursor_iter(editor)
	move_iter = cursor_iter.copy()
	move_iter.forward_char()
	cursor_iter.backward_char()

	# Remove matching brackets.

	editor.text_buffer.delete(cursor_iter, move_iter)

	# End recording for undo operation.

	editor.text_buffer.end_user_action()

	# Feedback to the statusbar indicating matching brackets have been removed.

	update_status_info(editor, msg0156, "yes")

	return