# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""
Main window and graphic user interface for the editor.

This modules contains code responsible for creating the editor's main window.
The editor's main window consists of a toolbar object containing 11 tool items,
a scroll window containing a textview and a status area comprising of three
statusbars.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from re import compile as recompile, UNICODE
from os import environ
from sys import setcheckinterval
from locale import getdefaultlocale

from gobject import timeout_add, source_remove, GError
from gtk import Window, keysyms, main, HBox, WIN_POS_CENTER
from gtk import window_set_auto_startup_notification
from gtk import STOCK_LEAVE_FULLSCREEN, Button, WINDOW_TOPLEVEL
from gtk.gdk import WINDOW_TYPE_HINT_NORMAL, WINDOW_STATE_MAXIMIZED
from gnome import init, PARAM_APP_DATADIR, PARAM_ENABLE_SOUND
from gnome.ui import libgnomeui_module_info_get
from gtkspell import Spell

from timeout import load_string_timeout, fullscreen_timeout
from timeout import garbage, hide_cursor_timeout
from utility import update_status_info
from utility import find_app_data_dir, create_button
from scribes_gconf import scribes_init_gconf, get_gconf_client
from accelerators import create_accel
from actions import create_actions
from maincontainer import create_main_container
from toolbar import create_toolbar
from scrollwindow import create_scroll_window
from textbuffer import create_text_buffer
from textview import create_text_view
from statusbar import create_statusbar
from cursor import cursor_position
from gotobar import create_gotobar
from findbar import create_findbar, remove_highlight_tag
from replacebar import create_replacebar
from modes import update_editor_mode
from internationalization import msg0022, msg0023, msg0024, msg0025


################################################################################
#
#							Editor Window
#
################################################################################


class Editor(Window):

	u"""
	Main class and graphic user interface properties of the editor.

	The Editor class defines the main graphic user interface of the application.
	The interface consists of a toolbar, a scroll window containing a textview
	and a status bar.

	"""

	HOMEDIR = environ["HOME"]
	DESKTOPDIR = HOMEDIR + "/Desktop"
	SCRIBESINFODIR = HOMEDIR + "/.gnome2/scribes"


	def __init__(self):
		"""Initialize and create Editor's main graphic user interface."""

		self.bookmark_list = []

		setcheckinterval(1)

		# Activate start up notification

		window_set_auto_startup_notification(True)

		# Initialize the GNOME libraries.

		props = {PARAM_APP_DATADIR: find_app_data_dir(),
		 		 PARAM_ENABLE_SOUND: True}

		init(u"scribes", u"0.1.0", properties=props)

		# Initialize the gconf monitoring deamons and get user configuration
		# values

		scribes_init_gconf(self)

		# Create editor's window.

		Window.__init__(self, WINDOW_TOPLEVEL)
		self.connect(u"delete-event", self.close_window_cb)
		self.connect(u"key-press-event", self.key_press_event_cb)
		self.connect(u"window-state-event", self.window_state_event_cb)

		# Set widget properties

		self.set_name(u"scribes_window")

		# Set container properties

		self.set_border_width(1)

		# msg0025, "Unsaved Document," is a string eligible for
		# internationalization and localization. See internationalization.py for
		# more details

		self.set_title(msg0025)
		self.set_role(u"main_window")
		self.set_type_hint(WINDOW_TYPE_HINT_NORMAL)
		self.set_icon_name(u"text-editor")
		self.set_default_size(700, 500)

		# Create accelerators for the editor

		accel = create_accel()
		self.add_accel_group(accel)

		# Create and initialize editor actions

		self.actiongroup = create_actions(self)

		# Create editor's main container

		self.main_container = create_main_container()
		self.add(self.main_container)

		# Toolbar area

		toolbar_area = HBox(homogeneous=False, spacing=0)

		# Create editor's toolbar

		toolbar = create_toolbar(self)
		toolbar_area.pack_start(toolbar, True, True, 0)
		self.main_container.pack_start(toolbar_area, False, False, 0)
		self.toolbar = toolbar
		toolbar.connect("enter-notify-event", self.enter_notify_event_cb)
		self.toolbar_area = toolbar_area

		# Create editor's scroll window

		scroll_window = create_scroll_window()
		self.main_container.pack_start(scroll_window, True, True, 0)
		self.scroll_window = scroll_window

		# Create editor's text buffer

		self.text_buffer = create_text_buffer(self)

		# Create editor's text view

		text_view = create_text_view(self)
		scroll_window.add(text_view)
		text_view.grab_focus()

		# Create editor's status area container

		status_area = HBox(homogeneous=False, spacing=0)
		self.main_container.pack_start(status_area, False, False, 0)

		self.status_area = status_area

		# Create editor's statusbar

		status_structure = create_statusbar()
		status_container = status_structure[0]
		status_area.pack_start(status_container, expand=True, fill=True,
								padding=0)

		self.message_id = ""
		self.status_one = status_structure[1]
		self.status_one_id = self.status_one.get_context_id("id-1")
		self.status_one.push(self.status_one_id, self.message_id)

		self.status_two = status_structure[2]
		self.status_two_id = self.status_two.get_context_id("id-2")
		self.status_two.push(self.status_two_id, "")

		self.status_three = status_structure[3]

		# Insert mode is the default editor mode. Let the status bar reflect
		# this mode by default.

		default_mode = u"INS"
		self.status_three_id = self.status_three.get_context_id(default_mode)
		self.status_three.push(self.status_three_id, default_mode.ljust(0))

		# Activate the statusbar's cursor position label

		cursor_position(self)

		# Call the garbage collector every 30 minutes.

		garbage_id = timeout_add(1800000, garbage, self)

		# Hide the cursor after 5 seconds of cursor inactivity

		cursor_id = timeout_add(5000, hide_cursor_timeout, self)

		# Spell Checking

		if self.spell_check:

			try:

				self.spell_checker = Spell(text_view, getdefaultlocale()[0])

			except GError:

				pass

################################################################################
#
#							Flags
#
################################################################################

		# This flag shows the file has already been saved and doesn't need
		# further saving. Used by the automatic saving function of the
		# editor. When the flag is true, the file is saved. Otherwise, the
		# file is unsaved.

		self.save_file_flag = True

		# There is no instance of the gotobar if the flag is false

		self.gotobar_flag = False

		# There is no instance of the findbar if the flag is false

		self.findbar_flag = False

		# There is no instance of the replacebar if the flag is false

		self.replacebar_flag = False

		# The loading_flag is used to determine whether or not the program
		# should load a file into the current buffer or should load the file in
		# a new instance of the editor

		self.loading_flag = False

		# Determines whether or not the currently open file is readonly

		self.readonly = False

		# Determines whether or not the file is a new file or an existing one

		self.isNewFile = True

		# Determines whether or not the buffer has been contaminated. A buffer
		# that is contaminated is one that has been modified at least once.

		self.dirty = False

		# Initialize the default encoding used decode and encode text files.

		self.encoding = "UTF-8"

		self.mode_flag = False

		self.mode_edit_flag = False

		self.progress_flag = False

		self.saving_flag = False

		self.fullscreen_flag = False

		self.toolbar_flag = True

		self.status_busy_flag = False

		self.char = ""

		self.cursor_pattern = recompile(r"\b\w+\b$", UNICODE)


################################################################################
#
#							Default File Name
#
################################################################################

		# The default file name is "Unsaved Document." The name is locale
		# dependent. As such, the string in eligible for internationalization
		# and localization.

		self.filename = msg0025


################################################################################
#
#								Modes
#
################################################################################

	def add_mode(self, icon="", progress=False):
		u"""
		Add an icon depicting user operations or editor's mode

		@param self: editor
		@type self: editor object

		@param icon: stock icon
		@typ icon: string

		@param progress: show/hide a progress bar
		@type progress: boolean
		"""

		# If the progress bar is up do not change icon modes. The progress bar
		# has a high priority.

		if self.progress_flag:

			return

		# If an icon already exist in the status area, remove it first.

		if self.mode_flag:

			self.remove_mode()

		# The flag indicates an icon exists in the status area

		self.mode_flag = True
		mode = update_editor_mode(self, icon, progress)

		# Update the status area and show icon mode

		self.status_area.pack_start(mode, expand=False, fill=False,
									padding=0)
		self.status_area.reorder_child(mode, 0)
		self.mode = mode
		mode.show()

		return


	def remove_mode(self):
		u"""
		Remove icon depicting user operations or editor's mode

		@param self: editor
		@type self: editor object

		"""

		if self.mode_flag:

			self.mode.hide()
			self.mode_flag = False
			self.mode.destroy()
			del self.mode

		return


	def reset_mode(self):

		# If the progress bar is up do not change icon modes. The progress bar
		# has a high priority.

		if self.progress_flag:

			return

		if self.readonly:

			self.add_mode("readonly")

			return

		if self.isNewFile and not self.dirty:

			self.remove_mode()

			return

		if not self.save_file_flag or self.mode_edit_flag:

			self.mode_edit_flag = False
			self.add_mode("edit")

			return

		else:

			self.add_mode("new")

		return


################################################################################
#
#								Gotobar
#
################################################################################

	def show_gotobar(self):
		u"""
		Construct and show the gotobar

		@param self: editor
		@type self: editor object
		"""

		self.add_mode("goto")
		self.gotobar_flag = True
		self.actiongroup.get_action(u"goto_line").set_sensitive(False)
		self.actiongroup.get_action(u"search_text").set_sensitive(False)
		self.actiongroup.get_action(u"replace_text").set_sensitive(False)
		self.gotobar = create_gotobar(self)
		self.main_container.pack_start(self.gotobar, False, False, 0)
		self.main_container.reorder_child(self.gotobar, 2)
		self.spin_button.grab_focus()
		self.gotobar.show_all()

		return

	def hide_gotobar(self):
		u"""
		Hide and destroy the gotobar

		@param self: editor
		@type self: editor object
		"""

		self.reset_mode()
		if self.readonly:

			flag = False

		else:

			flag = True

		self.gotobar_flag = False
		self.gotobar.hide_all()
		self.gotobar.destroy()
		self.actiongroup.get_action(u"goto_line").set_sensitive(True)
		self.actiongroup.get_action(u"search_text").set_sensitive(True)
		self.actiongroup.get_action(u"replace_text").set_sensitive(flag)
		self.text_view.grab_focus()

		return

################################################################################
#
#								Findbar
#
################################################################################


	def show_findbar(self):
		u"""
		Construct and show the findbar

		@param self: editor
		@type self: editor object
		"""

		self.add_mode("find")
		self.text_view.set_editable(False)
		self.findbar_flag = True
		self.actiongroup.get_action(u"goto_line").set_sensitive(False)
		self.actiongroup.get_action(u"search_text").set_sensitive(False)
		self.actiongroup.get_action(u"replace_text").set_sensitive(False)
		self.findbar = create_findbar(self)
		self.main_container.pack_start(self.findbar, False, False, 0)
		self.main_container.reorder_child(self.findbar, 2)
		self.entry.grab_focus()
		self.findbar.show_all()

		return


	def hide_findbar(self):
		u"""
		Hide and destroy the findbar

		@param self: editor
		@type self: editor object
		"""

		if self.readonly:

			flag = False

		else:

			flag = True

		self.reset_mode()
		self.findbar.hide_all()
		self.text_view.grab_focus()
		self.findbar_flag = False
		self.actiongroup.get_action(u"goto_line").set_sensitive(True)
		self.actiongroup.get_action(u"search_text").set_sensitive(True)
		self.actiongroup.get_action(u"replace_text").set_sensitive(flag)
		tag = self.text_buffer.get_tag_table().lookup(u"highlight")

		if tag:

			remove_highlight_tag(self)

		self.text_view.set_editable(True)
		self.text_view.queue_draw()
		self.findbar.destroy()
		timeout_add(7000, load_string_timeout, self)

		return


################################################################################
#
#								Replacebar
#
################################################################################


	def show_replacebar(self):
		u"""
		Construct and show the replacebar

		@param self: editor
		@type self: editor object
		"""

		self.replacebar_flag = True
		self.show_findbar()
		self.replacebar = create_replacebar(self)
		self.main_container.pack_start(self.replacebar, False, False, 0)
		self.main_container.reorder_child(self.replacebar, 3)
		self.replacebar.show_all()
		self.add_mode("replace")

		return


	def hide_replacebar(self):
		u"""
		Hide and destroy the replacebar

		@param self: editor
		@type self: editor object
		"""

		self.reset_mode()
		self.replacebar_flag = False
		self.hide_findbar()
		self.replacebar.hide_all()
		self.text_view.queue_draw()
		self.replacebar.destroy()

		return


################################################################################
#
#							Fullscreen Mode
#
################################################################################

	def activate_fullscreen(self, state=True):

		if state:

			# msg0024, "Leave fullscreen", is a string eligible for
			# internationalization and localization. See internationalization.py
			# for more details.

			button = create_button(STOCK_LEAVE_FULLSCREEN, msg0024)
			self.button = Button()
			self.button.connect("clicked", self.unfullscreen_cb)
			button.connect("enter-notify-event", self.enter_notify_event_cb)
			self.button.add(button)
			self.toolbar_area.pack_start(self.button, False, False, 0)
			self.toolbar_area.show_all()
			self.fullscreen()
			self.fullscreen_flag = True
			self.ft_id = timeout_add(5000, fullscreen_timeout, self)
			self.status_area.hide_all()

		else:

			source_remove(self.ft_id)
			self.button.hide_all()
			self.button.destroy()
			self.unfullscreen()
			self.fullscreen_flag = False
			self.toolbar_area.show_all()
			self.toolbar_flag = True
			self.status_area.show_all()

		return


################################################################################
#
#						Key Press Event Callback
#
################################################################################


	def key_press_event_cb(self, entry, key_press):
		u"""
		Callback to the editor's window "key-press-event"

		The "key-press-event" signal is emitted when the user presses a key on
		the	keyboard.

		@param key_press: A key press event.
		@type key_press: GTK Signal
		@param self: editor
		@type self: editor object
		"""

		# Listen for F1 key press to launch the Scribes manual

		if key_press.keyval == keysyms.F1:

			self.actiongroup.get_action(u"help").activate()

		# Listen for F12 key press to launch the preferences dialog

		if key_press.keyval == keysyms.F12:

			self.actiongroup.get_action(u"preferences").activate()

		# Listen for F11 to enter fullscreen mode

		if key_press.keyval == keysyms.F11:

			if not self.fullscreen_flag:

				self.activate_fullscreen()

			else:

				self.activate_fullscreen(False)

		# Hide and destroy the gotobar when ESC is pressed

		if key_press.keyval == keysyms.Escape and self.gotobar_flag:

			self.hide_gotobar()

		# Hide and destroy the findbar when ESC is pressed

		if key_press.keyval == keysyms.Escape and self.findbar_flag:

			self.hide_findbar()

		# Hide and destroy the replacebar when ESC is pressed

		if key_press.keyval == keysyms.Escape and self.replacebar_flag:

			self.hide_replacebar()

		# If self.completion_flag has not been defined yet an exception will
		# thrown. self.completion_flag is defined the first time the text
		# editor's word completion window is shown.

		try:

			# Leave the text editor's fullscreen mode. If the text editor's
			# word completion window is showing, do nothing. Nothing is done
			# because the Escape key should be used to hide the text editor's
			# completion window.

			if key_press.keyval == keysyms.Escape and self.fullscreen_flag \
				and not self.completion_flag:

				self.activate_fullscreen(False)

		except AttributeError:

			pass

		# Spell Checking

		if key_press.keyval == keysyms.F6:

			client = get_gconf_client()

			# We use this flag to fool and prevent the editor from switching to
			# edit mode when the user types into the buffer.

			self.mode_edit_flag = True

			if client.get_bool("/apps/scribes/spell_check"):

				client.set_bool("/apps/scribes/spell_check", False)

				# msg0022 is a string eligible for internationalization and
				# localization. See internationalization.py for me information.

				# Send feedback to user via statusbar

				update_status_info(self, msg0022, "spell")

			else:

				client.set_bool("/apps/scribes/spell_check", True)

				# msg0023 is a string eligible for internationalization and
				# localization. See internationalization.py for me information.

				# Send feedback to user via statusbar

				update_status_info(self, msg0023, "spell")

		return


################################################################################
#
#						Fullscreen Callback
#
################################################################################


	def unfullscreen_cb(self, widget):

		self.activate_fullscreen(False)

		return


	def enter_notify_event_cb(self, widget, event):


		if self.fullscreen_flag:

			source_remove(self.ft_id)
			self.toolbar_area.show_all()
			self.toolbar_flag = True

		return False


################################################################################
#
#						Close Window Callback
#
################################################################################


	def close_window_cb(self, widget, event):
		u"""
		Delete the editor's window.

		@param widget: the widget that received the signal
		@type widget: GTK Window
		@param event: the event that triggered the signal
		@type event: GTK Event
		@param data: the optional first user parameter specified with the
			connect method
		@type data: None

		@return: Returns False to propagate the signal to all widgets
		@rtype: boolean
		"""

		self.actiongroup.get_action(u"close_file").activate()

		return True


################################################################################
#
#						Window State Callback
#
################################################################################


	def window_state_event_cb(self, widget, event):

		if event.new_window_state == WINDOW_STATE_MAXIMIZED:

			self.maximize_flag = True

		else:

			self.maximize_flag = False

		return True


################################################################################
#
#							Main Loop
#
################################################################################


	def gui_mainloop(self):
		u"""
		Intialize the event loop used by all GTK applications

		@param self: editor
		@type self: editor object
		"""

		main()

		return
