# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""
This module contains functions for generating, manipulating, storing,
configuring and viewing various character encodings supported by the editor.
UTF-8 is the default and recommended character encoding supported by the editor.
However, the editor allows users to open and save files in several other
encodings of their choice.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""

from gobject import TYPE_BOOLEAN
from gtk import Label, HBox, combo_box_new_text, Dialog
from gtk import RESPONSE_CLOSE, WIN_POS_CENTER_ON_PARENT, STOCK_CLOSE, keysyms
from gtk import ListStore, TreeView, CellRendererToggle, TreeViewColumn
from gtk import CellRendererText, TREE_VIEW_COLUMN_AUTOSIZE
from gtk.gdk import WINDOW_TYPE_HINT_DIALOG, GRAVITY_CENTER

from gconf import VALUE_STRING
from scrollwindow import create_scroll_window
from scribes_gconf import get_gconf_client
from internationalization import msg0157, msg0158, msg0159, msg0160, msg0161
from internationalization import msg0162, msg0163


def key_press_event_cb(entry, key_press, dialog):
	u"""Callback to the dialog's "key-press-event"

	The "key-press-event" signal is emitted when the user presses a key on
	the	keyboard.

	@param entry:
	@type entry:

	@param key_press: Escape Key.
	@type key_press: A gtk.Event.

	@param dialog: A reference to the character encoding dialog.
	@type dialog: A gtk.Dialog object.

	"""

	# Close the character encoding dialog when the Escape key is pressed.

	if key_press.keyval == keysyms.Escape:

		dialog.response(RESPONSE_CLOSE)

	return


def generate_encoding_dictionary():
	u"""
	Generate a tuple containing supported encodings.

	@return: A tuple containing supported encodings.
	@rtype: A tuple object.

	"""

	encoding_dictionary = (

#	Codec 				Alias			Language

	("ASCII",			("us-ascii", "English")),
	("BIG5",			("csbig5", "Traditional Chinese")),
	("BIG5HKSCS",		("hkscs", "Traditional Chinese")),
	("CP037", 			("IBM039", "English")),
	("CP424", 			("IBM424", 	"Hebrew")),
	("CP437",	 		("437", "English")),
	("CP500",			("IBM500", "Western Europe")),
	("CP737",			("cp737", "Greek")),
	("CP775", 			("IBM775", "Baltic languages")),
	("CP850", 			("IBM850", "Western Europe")),
	("CP852",			("IBM852", "Central and Eastern Europe")),
	("CP855",			("IBM855", "Bulgarian, Macedonian, Russian, Serbian")),
	("CP856",			("cp856", "Hebrew")),
	("CP857",	 		("IBM857", "Turkish")),
	("CP860",			("IBM860", "Portuguese")),
	("CP861", 			("IBM861", "Icelandic")),
	("CP862", 			("IBM862", "Hebrew")),
	("CP863", 			("IBM863", "Canadian")),
	("CP864",			("IBM864", "Arabic")),
	("CP865",			("IBM865", "Danish, Norwegian")),
	("CP866", 			("IBM866", "Russian")),
	("CP869", 			("IBM869", "Greek")),
	("CP874", 			("cp874", "Thai")),
	("CP875", 			("cp875", "Greek")),
	("CP932", 			("mskanji", "Japanese")),
	("CP949", 			("uhc", "Korean")),
	("CP950",			("ms950", "Traditional Chinese")),
	("CP1006",			("cp1006", "Urdu")),
	("CP1026", 			("ibm1026", "Turkish")),
	("CP1140", 			("ibm1140", "Western Europe")),
	("CP1250", 			("windows-1250", "Central and Eastern Europe")),
	("CP1251", 			("windows-1251", "Bulgarian, Macedonian, Russian, Serbian")),
	("CP1252", 			("windows-1252", "Western Europe")),
	("CP1253", 			("windows-1253", "Greek")),
	("CP1254", 			("windows-1254", "Turkish")),
	("CP1255", 			("windows-1255", "Hebrew")),
	("CP1256", 			("windows1256", "Arabic")),
	("CP1257",			("windows-1257", "Baltic languages")),
	("CP1258", 			("windows-1258", "Vietnamese")),
	("EUC-JP", 			("ujis", "Japanese")),
	("EUC-JIS-2004", 	("eucjis2004", "Japanese")),
	("EUC-JISX0213", 	("eucjisx0213", "Japanese")),
	("EUC-KR", 			("ksc5601", "Korean")),
	("GB2312",			("euccn", "Simplified Chinese")),
	("GBK", 			("ms936", "Unified Chinese")),
	("GB18030", 		("gb18030-2000", "Unified Chinese")),
	("HZ", 				("hzgb", "Simplified Chinese")),
	("ISO2022-JP", 		("iso2022jp", "Japanese")),
	("ISO2022-jp-1", 	("iso2022jp-1", "Japanese")),
	("ISO2022-JP-2", 	("iso2022jp-2", "Japanese, Korean, Simplified Chinese")),
	("ISO2022-JP-2004", ("iso2022jp-2004", "Japanese")),
	("ISO2022-JP-3", 	("iso2022jp-3", "Japanese")),
	("ISO2022-JP-EXT", 	("iso2022jp-ext", "Japanese")),
	("ISO2022-KR", 		("csiso2022kr", "Korean")),
	("ISO8859-1", 		("iso-8859-1", "West Europe")),
	("ISO8859-2", 		("iso-8859-2", "Central and Eastern Europe")),
	("ISO8859-3", 		("iso-8859-3", "Esperanto, Maltese")),
	("ISO8859-4", 		("iso-8859-4", "Baltic languagues")),
	("ISO8859-5", 		("iso-8859-5", "Bulgarian, Macedonian, Russian, Serbian")),
	("ISO8859-6", 		("iso-8859-6", "Arabic")),
	("ISO8859-7", 		("iso-8859-7", "Greek")),
	("ISO8859-8", 		("iso-8859-8", "Hebrew")),
	("ISO8859-9", 		("iso-8859-9", "Turkish")),
	("ISO8859-10", 		("iso-8859-10", "Nordic languages")),
	("ISO8859-13", 		("iso-8859-13", "Baltic languages")),
	("ISO8859-14", 		("iso-8859-14", "Celtic languages")),
	("ISO8859-15", 		("iso-8859-15", "Western Europe")),
	("JOHAB", 			("cp1361", "Korean")),
	("KOI8-R", 			("koi8_r", "Russian")),
	("KOI8-U", 			("koi8_u", "Ukrainian")),
	("MAC-CYRILLIC", 	("maccyrillic", "Bulgarian, Macedonian, Russian, Serbian")),
	("MAC-GREEK", 		("macgreek", "Greek")),
	("MAC-ICELAND", 	("maciceland", "Icelandic")),
	("MAC-LATIN2", 		("maccentraleurope", "Central and Eastern Europe")),
	("MAC-ROMAN", 		("macroman", "Western Europe")),
	("MAC-TURKISH", 	("macturkish", "Turkish")),
	("PTCP154", 		("cp154", "Kazakh")),
	("SHIFT-JIS", 		("sjis", "Japanese")),
	("SHIFT-JIS-2004", 	("sjis2004", "Japanese")),
	("SHIFT-JISX0213", 	("sjisx0213", "Japanese")),
	("UTF-16", 			("utf16", "All languages")),
	("UTF-16-BE", 		("UTF-16BE", "All Languages (BMP only)")),
	("UTF-16-LE", 		("UTF-16LE", "All Languages (BMP only)")),
	("UTF-7", 			("U7", "All Languages")),
)

	return encoding_dictionary


def toggled_cb(cell, path, liststore):
	u"""
	Handles callback when a cell in the select column of the character encoding
	dialog is toggled.

	When a row in the select column is activated, we check to see if the cell
	is toggled on or off. If the cell is toggled on, we add the encoding to
	the GConf database, otherwise we remove the encoding from the GConf
	database.

	@param cell: A row indicating whether or not a character encoding is
				 selected.
	@type cell: A gtk.CellRendererToggle object.

	@param path: The full path on the selected row.
	@type path:

	@param liststore: A database containing all encodings supported by the
					  editor.
	@type liststore: A gtk.ListStore object.

	"""

	# Connect to GConf and get the encodings already selected by the user

	client = get_gconf_client()
	encoding_list = client.get_list("/apps/scribes/encoding", VALUE_STRING)

	# Toggle the check button in the select column

	liststore[path][0] = not liststore[path][0]

	# If the check button is toggled on, add the character encoding on that row
	# to GConf's encoding database, otherwise remove the encoding on that row
	# from GConf's encoding database.

	if liststore[path][0]:

		# Add liststore to gconf database

		encoding_list.append(liststore[path][1])
		client.set_list("/apps/scribes/encoding", VALUE_STRING, encoding_list)

	else:

		# Remove liststore from gconf database

		if liststore[path][1] in encoding_list:

			encoding_list.remove(liststore[path][1])
			client.set_list("/apps/scribes/encoding", VALUE_STRING,
							encoding_list)

	return


def create_encoding_view():
	u"""
	Create a treeview that contains supported encodings

	The generated treeview contains three columns, namely: the select, character
	encoding as well as language and region columns. The select column contains
	rows of toggle check buttons that allows a user to toggle on or off
	encoding selections. The character encoding column contains rows of mnemonic
	string representing character encodings. The language column contains rows
	of string associating character encodings to regions and languages.

	@return: A treeview containing supported encodings
	@rtype: A gtk.TreeView object

	"""

	# Connect to GConf and get the encodings already selected by the user

	client = get_gconf_client()
	encoding_list = client.get_list("/apps/scribes/encoding", VALUE_STRING)

	# Generate the encoding dictionary.

	encoding_dictionary = generate_encoding_dictionary()

	# Create a treeview to house generated encodings.

	encoding_view = TreeView()

	# Create a liststore to serve as the database for supported encodings.

	liststore = ListStore(TYPE_BOOLEAN, str, str)

	# Populate the data store with supported encodings.

	for value in encoding_dictionary:

		if value[0] in encoding_list:

			select = True

		else:

			select = False

		liststore.append([select, value[0], value[1][1]])

	# Create the renderer for the select column.

	renderer = CellRendererToggle()
	renderer.connect(u"toggled", toggled_cb, liststore)
	renderer.set_property(u"activatable", True)

	# Create a column for selecting encodings.

	column = TreeViewColumn(msg0163, renderer)
	encoding_view.append_column(column)
	column.set_expand(False)
	column.set_sizing(TREE_VIEW_COLUMN_AUTOSIZE)
	column.add_attribute(renderer, u"active", 0)

	# Create a renderer for the character encoding column.

	renderer = CellRendererText()

	# Create a column for character encoding.

	column = TreeViewColumn(msg0162, renderer, text=1)
	encoding_view.append_column(column)
	column.set_expand(True)
	column.set_sizing(TREE_VIEW_COLUMN_AUTOSIZE)

	# Create the renderer for the Language column

	renderer = CellRendererText()

	# Create a column for Language and Region and set the column's properties.

	column = TreeViewColumn(msg0161, renderer, text=2)
	encoding_view.append_column(column)
	column.set_expand(True)
	column.set_sizing(TREE_VIEW_COLUMN_AUTOSIZE)

	# Set treeview properties

	encoding_view.set_model(liststore)
	encoding_view.columns_autosize()
	encoding_view.set_enable_search(True)
	encoding_view.set_search_column(1)
	encoding_view.set_property("rules-hint", True)

	return encoding_view


def show_encoding_dialog(editor):
	u"""
	Show the encoding dialog.

	The encoding dialog presents a list of encodings supported by the editor.
	Encodings selected from the dialog are added to the combobox list.
	Technically, encoding dialog is just a front end to Gconf. Selected items
	from the encoding dialog are stored in the GConf database. When selections
	occur, the database is updated and so in the combobox list.

	@param editor:
	@type editor:

	"""
	# Create a dialog object.

	dialog = Dialog()
	dialog.connect(u"key_press_event", key_press_event_cb, dialog)

	# Set window properties

	dialog.set_title(msg0160)
	dialog.set_role(u"encoding_dialog")
	dialog.activate_default()
	dialog.set_transient_for(editor)
	dialog.set_type_hint(WINDOW_TYPE_HINT_DIALOG)
	dialog.set_modal(True)
	dialog.set_size_request(640, 480)
	dialog.set_skip_taskbar_hint(True)
	dialog.set_icon_name(u"text-editor")
	dialog.set_position(WIN_POS_CENTER_ON_PARENT)
	dialog.set_resizable(True)
	dialog.set_has_separator(False)
	dialog.set_gravity(GRAVITY_CENTER)

	scrollwindow = create_scroll_window()
	scrollwindow.add(create_encoding_view())
	dialog.vbox.pack_start(scrollwindow, True, True, 1)

	# Set dialog properties

	dialog.add_button(STOCK_CLOSE, RESPONSE_CLOSE)

	dialog.show_all()
	response = dialog.run()

	if response:

		dialog.hide_all()
		dialog.destroy()

	return


def update_combobox_list(combobox):
	u"""
	Updates the the open dialog combobox with encodings selected by the user.

	@param combobox: Combobox containing a list of selected encodings.
	@type combobox: A gtk.ComboBox object

	"""

	# Connect to GConf and get the encodings already selected by the user

	client = get_gconf_client()
	encoding_list = client.get_list("/apps/scribes/encoding", VALUE_STRING)

	# Remove all items in the combobox list.

	combobox.get_model().clear()

	# Insert first item into the combobox list.

	combobox.append_text(msg0159)

	# Insert selected encodings into the combobox list.

	if encoding_list:

		for encoding in encoding_list:

			combobox.append_text(encoding)

	# Insert last item into the combobox list.

	combobox.append_text(msg0158)

	# If the combobox is for the save dialog box, show the current encoding of
	# file as the active text. If the combobox is for the open dialog box, then
	# show the recommended encoding as the active text.

	if combobox.box_type == "save":

		if combobox.encoding:

			if combobox.encoding in encoding_list:

				model = combobox.get_model()

				count = 0

				while True:

					if combobox.encoding == model[count][0]:

						combobox.set_active(count)
						break

					count += 1

			else:

				combobox.set_active(0)

		else:

			combobox.set_active(0)

	else:

		combobox.set_active(0)

	return


def combobox_cb(combobox, editor):
	u"""
	Handles callback when items in combobox changes.

	@param combobox: A reference to the combobox object holding a list of
					 encodings
	@type combobox: A GTK ComboBox object

	"""

	# If the "Add or Remove" item is selected in the ComboBox, show the encoding
	# dialog. Otherwise set the encoding of the editor to the value of the
	# active text in the combobox list.

	try:

		if combobox.get_active_text() == msg0158:

			show_encoding_dialog(editor)
			update_combobox_list(combobox)

		elif combobox.get_active_text() == msg0159:

			editor.encoding = "UTF-8"

		else:

			editor.encoding = combobox.get_active_text()

	except AttributeError:

		pass

	return


def combobox_changed_cb(client, cnxn_id, entry, combobox):
	u"""
	Handles callback when user makes changes to editor's margin display

	The notify function is invoked when the value of a key changes.

	client -- gconf client
	cnxn_id -- connection id from notification daemon
	entry --
	editor -- editor object

	"""

	update_combobox_list(combobox)

	return


def create_encoding_box(editor, box_type="open"):
	u"""
	Create a container to house the encoding label and combobox on the open
	dialog.

	@return: A container holding an encoding label and combobox
	@rtype: A HBox container a Label and ComboBox

	"""

	# Connect to GConf.

	client = get_gconf_client()

	# Create an encoding box

	encoding_box = HBox()

	# Add a label titled "Character Encoding"

	encoding_label = Label(msg0157)
	encoding_label.set_use_underline(True)
	encoding_box.pack_start(encoding_label, False, True, 0)

	# Add a combobox to hold available encodings

	combobox = combo_box_new_text()
	combobox.encoding = editor.encoding
	combobox.box_type = box_type
	combobox.connect(u"changed", combobox_cb, editor)
	encoding_box.pack_start(combobox, True, True, 0)
	update_combobox_list(combobox)
	client.notify_add("/apps/scribes/encoding", combobox_changed_cb, combobox)

	return encoding_box
