# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


u"""
File loading module for Scribes

This module describes a class that performs operations necessary to load a
file. This module also exposes a launch_new_file_timeout() callback. The
callback is activated only when more than one file is being loaded. When only
one file is being loaded, the callback is ignored.

@note:	The text editor follows a document centric and spatial paradigm. This
		means each file is loaded in its own editor window. Conceptually, think
		of the editor windows as a document, rather than an application.

"""

from gc import collect
from os import path, environ, R_OK, W_OK, F_OK, access

from gobject import timeout_add
from gtk import main as gmain, quit_add
from gnomevfs import format_uri_for_display, get_file_info, get_mime_type
from gnomevfs.async import open as gvfsopen

from syntax import syntax_highlight
from error import error_dialog
from utility import show_status_info
from timeout import drag_and_drop_timeout, activate_progressbar_timeout
from position import window_position
from internationalization import msg0025, msg0030, msg0031, msg0032, msg0033
from internationalization import msg0034, msg0035, msg0036, msg0037, msg0153
from cursor import show_cursor

import main


def launch_new_file_timeout(filename):
	u"""
	Handles a timeout function called to launch a new editor after 2 seconds

	@param filename:	name of a file
	@type filename: 	string

	@return:			False to ensure the function is called only once
	@rtype: 			boolean

	"""

	main.launch_new_file(filename, False)

	return False


class FileLoader(object):
	u"""
	Defines attributes and methods necessary to load a file.

	The sole function of this class is to load a text file into the editor's
	buffer. The	class exposes methods that performs numerous checks to ensure
	only text files are loaded into the editor's buffer. Hence, only text files
	that the user has permission to view can be loaded into the editor's buffer.

	"""

	def __init__(self, editor, encoding="UTF-8"):
		u"""
		Initialize a FileLoader instance

		@param self: Reference to a FileLoader instance
		@type self: a FileLoader object

		@param editor: 	Reference to the editor instance
		@type editor: An editor object

		"""

		# A list of file the text editor should not open.

		self.blacklist = ("pdf", "jpeg", "png", "rtf", "jpg", "ogg", "mov",
						  "mp3", "mpej", "avi", "asf", "wav" ,"chm", "doc",
						  "exe", "sxw", "odt", "tar", "gz", "bz2", "bz",
						  "iso", "ram")

		# Get system encodings

		self.encoding = encoding
		self.editor = editor

		# Show the busy cursor while loading a file.

		show_cursor(self.editor, "busy")
		self.iter = None

		# Initialize error messages

		self.message_id = None

		# The progress_flag is True when the progress bar is activated and
		# false otherwise. The progress bar is activated when a file takes more
		# than a second to load.

		self.editor.progress_flag = False

		# During a drag and drop operation, the "drag-data-recieved" signal
		# gets called twice. The reason for this is unclear to me. We block the
		# signal after it is emitted the first time, to prevent it from being
		# emitted again. The drag_flag is true if a file is loaded via drag and
		# drop and false otherwise.
		#
		# The try block section is only relevant during a drag and drop
		# operation. Therefore, an exception is always thrown and caught during
		# normal loading.

		try:

			if self.editor.drag_flag:

				self.editor.text_view.handler_block(self.editor.txtid)

		except AttributeError:

			pass


	def check_load_flag(self, filename):
		u"""
		Determines whether or not the file should be loaded in a new window

		Each document is loaded in its own window. So if a window already has
		a file loaded, a new editor window is created and the new file is loaded
		into the newly created window.

		@param self: Reference to a FileLoader instance.
		@type self: A FileLoader object.

		@param filename: The name of the file to be loaded.
		@type filename: A string.

		"""

		# If the buffer is not empty, or the buffer has been modified, load the
		# file in a new editor instance.

		if self.editor.text_buffer.get_char_count() or \
			self.editor.text_buffer.get_modified() or self.editor.loading_flag \
			or self.editor.filename !=  msg0025:

			show_cursor(self.editor)
			quit_add(0, gmain)
			timeout_add(2, launch_new_file_timeout, filename)

			try:

				if self.editor.drag_flag:

					timeout_add(11, drag_and_drop_timeout, self.editor)

			except AttributeError:

				pass

			return

		self.editor.loading_flag = True
		self.redefine_filename(filename)

		return


	def redefine_filename(self, filename):
		u"""
		Define the file's URI, filename and statusbar name

		@param self: Reference to a FileLoader instance.
		@type self: A FileLoader object.

		@param filename: The name of the file to be loaded.
		@type filename: A string.

		"""

		self.editor.file_uri = str(filename).strip()
		temp = str(format_uri_for_display(self.editor.file_uri))
		self.editor.filename = temp.strip()

		if self.editor.filename.startswith(environ[u'HOME']):

			filename = str(self.editor.filename.replace(environ[u'HOME'], "~"))
			self.editor.prettyname = filename.strip()

		else:

			self.editor.prettyname = str(self.editor.filename).strip()

		self.check_file_validity(self.editor.filename)

		return


	def check_file_validity(self, filename):
		u"""
		Check file permissions, type and existence

		@param self: Reference to a FileLoader instance.
		@type self: A FileLoader object.

		@param filename: The name of the file to be loaded.
		@type filename: A string.

		"""



		# Check to see if the file exists.

		if not access(filename, F_OK):

			# msg0037 is missing file error message. It is a string eligible for
			# internationalization and localization. See internationalization.py
			# for more details.

			show_cursor(self.editor)
			error_dialog(self.editor, msg0037)
			self.error_recovery()

			return

		# Check to see if the file is a text file.

		if not path.isfile(filename):

			# msg0036 is a file check error message. It is a string eligible for
			# internationalization and localization. See internationalization.py
			# for more details.

			show_cursor(self.editor)
			error_dialog(self.editor, msg0036)
			self.error_recovery()

			return

		#

		filepath, ext = path.splitext(filename)

		if not ext:

			pass

		elif ext in self.blacklist:

			show_cursor(self.editor)
			error_dialog(self.editor, msg0036)
			self.error_recovery()

			return


		# Check read file permissions.

		if not access(filename, R_OK):

			# msg0035 is an error message regarding permission problems. It is
			# a string eligible for internationalization and localization. See
			# internationalization.py for more details.

			show_cursor(self.editor)
			error_dialog(self.editor, msg0035)
			self.error_recovery()

			return

		# Check the mimetype of the file. If the mimetype is not text,
		# do extension checks.

		mimetype = get_mime_type(self.editor.file_uri)

		if not mimetype:

			pass

		elif not mimetype.startswith("text/"):

			filepath, ext = path.splitext(filename)

			if ext in self.blacklist:

				show_cursor(self.editor)
				error_dialog(self.editor, msg0036)
				self.error_recovery()

				return

		# Check write file permissions

		if not access(filename, W_OK) or self.editor.readonly:

			self.prepare_editor_for_loading(readonly=True)

		else:

			self.prepare_editor_for_loading()

		return


	def prepare_editor_for_loading(self, readonly=False):
		u"""
		Set up the editor window, retrieve basic information about the file,
		check for unicode validity.

		A GTK Textbuffer only accepts UTF-8 characters, so we need to convert
		characters in a file to UTF-8 before loading it to the editor's buffer.

		@param self: Reference to a FileLoader instance.
		@type self: A FileLoader object.

		@param readonly: 	whether or not file is readonly
		@type readonly: 	boolean

		"""

		if readonly:

			self.editor.readonly = True

			# msg0034, "Readonly," is a string eligible for internationalization
			# and localization. See internationalization.py for more details.

			self.editor.set_title(path.basename(self.editor.filename)+ msg0034)
			self.editor.text_view.set_editable(False)
			self.editor.actiongroup.get_action(u"saveas_file").set_sensitive(False)
			self.editor.actiongroup.get_action(u"replace_text").set_sensitive(False)
			self.editor.text_view.set_show_line_numbers(False)
			self.editor.text_view.set_highlight_current_line(False)
			self.editor.text_view.set_cursor_visible(False)
			self.editor.text_view.set_show_margin(False)


			self.editor.status_three.pop(self.editor.status_three_id)
			string = u" "
			self.editor.status_three_id = self.editor.status_three.get_context_id(string)
			self.editor.status_three.push(self.editor.status_three_id, string)
			self.editor.text_view.handler_block(self.editor.ovrid)

			self.editor.status_two.pop(self.editor.status_two_id)
			string = u" "
			self.editor.status_two_id = self.editor.status_two.get_context_id(string)
			self.editor.status_two.push(self.editor.status_two_id, string)
			self.editor.text_buffer.handler_block(self.editor.markid)
			self.editor.text_buffer.handler_block(self.editor.marksetid)

		else:

			self.editor.set_title(path.basename(self.editor.filename))


		# Basic File Information

		file_info = get_file_info(self.editor.file_uri)
		self.editor.file_size = file_info.size
		self.editor.opt_size = file_info.io_block_size
		self.editor.opt_count = 0.0

		# Determine utf-8 strictness for text decoding. If the file is a text
		# use "replace", otherwise use "strict."

		#mime = get_mime_type(self.editor.file_uri)

		self.load_file()

		return


	def load_file(self):
		u""""
		Load the file into the editor's buffer.

		@param self: Reference to a FileLoader instance.
		@type self: A FileLoader object.

		@note: Loading a file into the buffer is an asynchronous operation.

		"""

		# Block the "changed" and "modified changed" text buffer signals. These
		# signals should be blocked to prevent the callbacks associated with
		# them from being activated when loading a file.

		self.editor.text_buffer.handler_block(self.editor.tb_mod_id)
		self.editor.text_buffer.handler_block(self.editor.tb_chg_id)
		self.editor.text_buffer.handler_block(self.editor.tb_ins_id)

		try:

			# Hide the cursor and highlight line when loading files into the
			# editor's buffer.

			if self.editor.text_view.get_cursor_visible():

				self.editor.text_view.set_cursor_visible(False)
				self.editor.text_view.set_highlight_current_line(False)

			# Show the open statusbar icon

			self.editor.add_mode("open")

			# Open the file for loading. This operation is asynchronous.

			gvfsopen(self.editor.file_uri, self.open_cb)

			# Activate the progress bar timer. If the file is still loading
			# after a second, a progress bar is activated in the status bar
			# area.

			self.editor.gtimer = timeout_add(1000, activate_progressbar_timeout,
									 self.editor)

		except:

			# msg0033 is an error message. It is a string eligible for
			# internationalization and localization. See internationalization.py
			# for more details

			show_cursor(self.editor)
			error_dialog(self.editor, msg0033)
			self.error_recovery()

		else:

			# Recall the files window state and position if the file has an
			# entry in Scribes' database.

			window_position(self.editor, "set", self.editor.filename)

			# Make some toolbar buttons insensitive when the file is loading.

			self.editor.actiongroup.get_action(u"saveas_file").set_sensitive(False)
			self.editor.actiongroup.get_action(u"replace_text").set_sensitive(False)
			self.editor.actiongroup.get_action(u"print_file").set_sensitive(False)
			self.editor.actiongroup.get_action(u"goto_line").set_sensitive(False)
			self.editor.actiongroup.get_action(u"search_text").set_sensitive(False)
			self.editor.actiongroup.get_action(u"preferences").set_sensitive(False)
			self.editor.isNewFile = False

		return


	def open_cb(self, handle, exec_type):
		u"""
		Handles callback when a file is being open for reading

		@param self: reference to a FileLoader instance
		@type self: a FileLoader object
		@param handle: file handle
		@type handle: ?
		@param exec_type: ?
		@type exec_type: ?

		"""

		# msg0032, "Loading file please wait...," is a string eligible for
		# internationalization and localization. See internationalization.py
		# for more details

		self.editor.text_buffer.begin_not_undoable_action()
		self.editor.text_buffer.set_text(u"")
		self.iter = self.editor.text_buffer.get_start_iter()
		self.editor.text_view.set_editable(False)
		handle.read(self.editor.opt_size, self.read_cb)

		# Feedback to user via statusbar

		show_status_info(self.editor, msg0032)

		return


	def read_cb(self, handle, textbuffer, exc_type, bytes_requested):

		u"""
		Handles callback when a file is being opened for reading

		@param self: reference to a FileLoader instance
		@type self: a FileLoader object
		@param handle: file handle
		@type handle: ?
		@param buffer: ?
		@type buffer: ?
		@param exc_type: ?
		@type exc_type: ?
		@param bytes_requested: ?
		@type bytes_requested: ?

		"""

		if textbuffer != "":

			# Progress bar information

			self.editor.opt_count = self.editor.opt_count + self.editor.opt_size

			try:

				# Decode strings in text file to unicode strings

				unicode_txt = textbuffer.decode(self.encoding, "strict")

			except UnicodeDecodeError:

				show_cursor(self.editor)
				self.editor.progress_flag = False
				error_dialog(self.editor, msg0031)
				self.error_recovery()

				return

			try:

				# Encode to UTF-8 because gtk.Textbuffer uses UTF-8

				output = unicode_txt.encode("utf8")

			except UnicodeEncodeError:

				show_cursor(self.editor)
				self.editor.progress_flag = False
				error_dialog(self.editor, msg0153)
				self.error_recovery()

				return

			# Insert character in the buffer.

			self.editor.text_buffer.insert(self.iter, output)
			self.iter = self.editor.text_buffer.get_end_iter()
			handle.read(self.editor.opt_size, self.read_cb)


		else:

			self.editor.progress_flag = False
			start_iter = self.editor.text_buffer.get_start_iter()

			# If the file has an entry in Scribes' database, place the cursor at
			# the file's last cursor position before it was closed.

			if self.editor.position_flag:

				line_iter = self.editor.text_buffer.get_iter_at_line(self.editor.pos[0][0]-1)
				line_iter.set_line_index(self.editor.pos[0][1])
				self.editor.text_buffer.place_cursor(line_iter)
				cursor_mark = self.editor.text_buffer.get_insert()
				self.editor.text_view.scroll_to_mark(cursor_mark, 0.3,
								 use_align=False, xalign=0.5,
								 yalign=0.5)

			else:

				self.editor.text_buffer.place_cursor(start_iter)

			self.editor.text_view.queue_draw()

			# Activate syntax highlight.

			syntax_highlight(self.editor, self.editor.file_uri)

			# Make toolbar buttons sensitive again.

			self.editor.actiongroup.get_action(u"saveas_file").set_sensitive(True)
			self.editor.actiongroup.get_action(u"print_file").set_sensitive(True)
			self.editor.actiongroup.get_action(u"goto_line").set_sensitive(True)
			self.editor.actiongroup.get_action(u"search_text").set_sensitive(True)
			self.editor.actiongroup.get_action(u"preferences").set_sensitive(True)

			# If the file is not in "readonly" mode, show the cursor and line
			# highlight. Also make the replace button sensitive.

			if not self.editor.readonly:

				self.editor.text_view.set_cursor_visible(True)
				self.editor.text_view.set_highlight_current_line(True)
				self.editor.actiongroup.get_action(u"replace_text").set_sensitive(True)

			# msg0030, "Loaded file %s," is a string eligible for
			# internationalization and localization. See internationalization.py
			# for more details.

			self.message_id = msg0030 % self.editor.prettyname
			show_status_info(self.editor, self.message_id, timeout=True)

			# Make the buffer editable if the file is not in "readonly" mode.

			if not self.editor.readonly:

				self.editor.text_view.set_editable(True)

			# Reset the buffer modification status to False.

			self.editor.text_buffer.set_modified(False)

			# Unblock the "changed" and "modified changed" signals.

			self.editor.text_buffer.handler_unblock(self.editor.tb_mod_id)
			self.editor.text_buffer.handler_unblock(self.editor.tb_chg_id)
			self.editor.text_buffer.handler_unblock(self.editor.tb_ins_id)
			self.editor.text_buffer.end_not_undoable_action()

			# The flag shows the loading process is over.

			self.editor.loading_flag = False

			# The flag indicates the buffer has been tainted.

			self.editor.dirty = True

			# The drag flag is only relevant during a drag and drop operation.

			try:

				if self.editor.drag_flag:

					timeout_add(11, drag_and_drop_timeout, self.editor)

			except:

				pass

			# Close the open file

			handle.close(self.close_cb)

		return


	def close_cb(self, *args):
		u"""
		Handles callback to close an open file

		@param self:	fileloader object

		"""

		self.editor.encoding = self.encoding

		show_cursor(self.editor)

		# Call the garbage collector

		collect()

		return


	def error_recovery(self):
		u"""
		Reset the editor to a neutral state after an error occurs

		This function is called when a error occurs while a file can not be
		loaded for several reasons. The function returns the editor window to a
		neutral state as if the a new window has just been launched.

		@param self:	fileloader object

		"""

		if not self.editor.text_view.get_cursor_visible():

			self.editor.text_view.set_cursor_visible(True)
			self.editor.text_view.set_highlight_current_line(True)

		self.editor.remove_mode()
		self.editor.text_buffer.set_text(u"")
		self.editor.text_view.set_editable(True)
		self.editor.actiongroup.get_action(u"saveas_file").set_sensitive(True)
		self.editor.actiongroup.get_action(u"print_file").set_sensitive(True)
		self.editor.actiongroup.get_action(u"goto_line").set_sensitive(True)
		self.editor.actiongroup.get_action(u"search_text").set_sensitive(True)
		self.editor.actiongroup.get_action(u"replace_text").set_sensitive(True)
		self.editor.actiongroup.get_action(u"preferences").set_sensitive(True)
		self.editor.text_buffer.handler_unblock(self.editor.tb_mod_id)
		self.editor.text_buffer.handler_unblock(self.editor.tb_chg_id)
		self.editor.text_buffer.handler_unblock(self.editor.tb_ins_id)
		self.editor.text_buffer.begin_not_undoable_action()
		self.editor.text_buffer.end_not_undoable_action()
		self.editor.text_buffer.set_modified(False)
		self.editor.set_title(msg0025)
		self.editor.filename = msg0025
		self.editor.loading_flag = False
		self.editor.readonly = False
		self.editor.isNewFile = True
		self.editor.dirty = False
		self.editor.save_file_flag = True
		show_status_info(self.editor, u" ")

		try:

			if self.editor.drag_flag:

				timeout_add(11, drag_and_drop_timeout, self.editor)

		except AttributeError:

			pass

		return