# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""
Gotobar for the text editor.

The gotobar is a widget that appears just above the status bar that allows a
user move the cursor to a specific line.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""
from gtk import Label, Adjustment, HBox, SpinButton
from gtk import UPDATE_IF_VALID, keysyms, Image, STOCK_JUMP_TO, ICON_SIZE_BUTTON

from internationalization import msg0055, msg0056


def value_changed_cb(spin_button, editor):
	u"""
	Handles callback when the entry in the spinbutton object changes.

	@param spin_button: Reference to the spinbutton object on the gotobar.
	@type spin_button: A gtk.SpinButton object

	@param editor: Reference to an editor instance
	@type editor: An editor object

	"""

	# Get the line number

	line = spin_button.get_value() - 1

	# Adjust the cursor and text view as per changes made to the spinbutton.

	line_iter = editor.text_buffer.get_iter_at_line(int(line))
	editor.text_buffer.place_cursor(line_iter)
	cursor_mark = editor.text_buffer.get_insert()
	editor.text_view.scroll_to_mark(cursor_mark, 0.3,
								 use_align=False, xalign=0.5,
								 yalign=0.5)

	return


def key_press_event_cb(spin_button, key_press, editor):
	u"""
	Handles callback when the return key is hit.

	Hitting the return or enter key has the side effect of hiding the gotobar.

	@param spin_button: Reference to the spinbutton object on the gotobar.
	@type spin_button: A gtk.SpinButton object

	@param key_press: Any keypress event
	@type key_press: A gtk.gdk.Event object

	@param editor: Reference to an editor instance
	@type editor: An editor object

	"""

	# Hides the gotobar when the return key is pressed

	if key_press.keyval == keysyms.Return:

		editor.hide_gotobar()

	return


def create_gotobar(editor):
	u"""
	Create the editor's gotobar.

	The gotobar bar is widget that appears just above the statusbar.

	@param editor: Reference to an editor instance
	@type editor: An editor object

	@return: A gotobar containing labels and spinbutton entry widget
	@rtype: A gtk.Hbox object

	"""

	editor = editor
	gotobar = HBox(False, 0)

	# Add the GTK JUMP TO stock icon to the gotobar widget

	image = Image()
	image.set_from_stock(STOCK_JUMP_TO, ICON_SIZE_BUTTON)
	gotobar.pack_start(image, False, False, 3)

	# Add a label to the gotobar widget
	# See internationalization.py for full message of msg0055

	label = Label(msg0055)
	gotobar.pack_start(label, False, False, 3)

	# Get the current line number

	cursor_mark = editor.text_buffer.get_insert()
	cursor_iter = editor.text_buffer.get_iter_at_mark(cursor_mark)
	cursor_line = cursor_iter.get_line()

	# Create an adjustment for the spin button below

	adjustment = Adjustment()

	value = cursor_line + 1
	lower = 1
	upper =  editor.text_buffer.get_line_count()
	step_increment = 1
	page_increment = 5
	page_size = 11

	adjustment.set_all(value, lower, upper, step_increment, page_increment,
					   page_size)

	# Add a spin button indicating the current line number to the gotobar
	# widget. The user can enter line numbers manually, or can use the spin
	# button to jump them to the specified line.

	spin_button = SpinButton(adjustment)
	editor.spin_button = spin_button
	spin_button.connect("value-changed", value_changed_cb, editor)
	spin_button.connect("key-press-event", key_press_event_cb, editor)
	gotobar.pack_start(spin_button, False, False, 3)
	spin_button.set_numeric(True)
	spin_button.set_update_policy(UPDATE_IF_VALID)
	spin_button.set_wrap(False)
	spin_button.set_snap_to_ticks(True)
	spin_button.set_digits(0)
	spin_button.set_property("xalign", 1)

	# Add a label indicating the total number of lines in the buffer
	# See internationalization.py for full message of msg0056

	gotobar_label = Label(msg0056 % upper)
	gotobar.pack_start(gotobar_label, False, False, 3)

	return gotobar
