# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""
This modules contains indentation and space operations for the text editor.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""

from gtk.gdk import beep

from utility import response, update_status_info
from internationalization import msg0059, msg0060, msg0134, msg0061
from internationalization import msg0135, msg0141, msg0142, msg0143, msg0144
from internationalization import msg0146, msg0147, msg0148
from cursor import show_cursor


def shift(editor, line, direction="right"):
	u"""
	Shift all characters on a line left or right one tab space.

	@param editor: Reference to the editor instance
	@type editor: An editor object

	@param line: The line number to be operated upon.
	@type line: An integer object

	@param direction: Direction to shift characters on a line. Direction can
					  either be left of right.
	@type direction: A string object

	@return: The result of the shift operation. True if operation is successful,
			 false otherwise.
	@rtype: A boolean object

	"""

	# Initialize the result to false.

	result = False

	# Get the iterator at the beginning of the line

	start_iter = editor.text_buffer.get_iter_at_line(line)

	# If direction is right, simply add a tab character to the beginning of
	# the line. If direction is left, and the first character is a tab, remove
	# it. If direction is left and the first character is spaces, remove all
	# spaces or tab characters less than, or equal, in length to the tab width
	# size.

	if direction == "right":

		editor.text_buffer.insert(start_iter, u"\t")
		result = True

	else:

		end_iter = start_iter.copy()
		tab_size = editor.text_view.get_tabs_width()

		# Used to keep track of how many spaces end_iter has moved.

		count = 0

		# If the first character is a space or tab, iterate through the line.

		while end_iter.get_char() == " " or end_iter.get_char() == "\t":

			response()
			result = True

			# If the first character is a tab, we are done. Move end_iter one
			# character forward and exit the loop

			if end_iter.get_char() == "\t" and not count:

				end_iter.forward_char()
				break

			# If character is a space, increase count by one and move
			# end_iter one space character forward.

			if end_iter.get_char() == " ":

				count += 1
				end_iter.forward_char()

				# If count is greater than the tab width size, exit the loop,
				# otherwise, move to the beginning of the loop

				if count >= tab_size:

					break

				else:

					continue

			# If character is a tab, increase count by tab width size and move
			# end_iter one tab character forward.

			if end_iter.get_char() == "\t":

				count += tab_size
				end_iter.forward_char()

				# If count is greater than the tab width size, exit the loop,
				# otherwise, move to the beginning of the loop

				if count >= tab_size:

					break

				else:

					continue

		# Delete space and/or tab characters at the beginning of the line.

		if result:

			editor.text_buffer.delete(start_iter, end_iter)

	return result


def indent(editor):
	u"""
	Indent a line or selection of lines

	@param editor: Reference to the editor instance
	@type editor: An editor object

	"""

	show_cursor(editor, "busy")

	# Begin recording for undo operation.

	editor.text_buffer.begin_user_action()

	# Make the buffer uneditable to prevent interruption and possible corruption
	# off iterators by the user.

	editor.text_view.set_editable(False)

	# We use this flag to fool and prevent the editor from switching to edit
	# mode when the user types into the buffer.

	editor.mode_edit_flag = True

	# Retrieve the cursor line.

	cursor_mark = editor.text_buffer.get_insert()
	cursor_iter = editor.text_buffer.get_iter_at_mark(cursor_mark)
	cursor_line = cursor_iter.get_line()

	# Get selection boundaries

	selection = editor.text_buffer.get_selection_bounds()

	# If there is no selection indent the line anyway.

	if not selection:

		# String to push to the statusbar

		message = msg0146 % (cursor_line + 1)
		shift(editor, cursor_line, direction="right")
		update_status_info(editor, message, u"yes")

	else:

		start_line = selection[0].get_line()
		end_line = selection[1].get_line()

		# If the selection is on the same line, indent the line

		if start_line == end_line:

			# String to push to the statusbar

			message = msg0146 % (start_line + 1)
			shift(editor, start_line, direction="right")
			update_status_info(editor, message, u"yes")

		else:

			# Iterate through each line that is selected, and add a tab
			# character to its beginning.

			for lines in range(start_line, end_line+1):

				start_iter = editor.text_buffer.get_iter_at_line(lines)

				# If there are no characters on the line, move to the next line.

				if start_iter.ends_line():

					continue

				result = shift(editor, lines, direction="right")

			# The result is always true, for now.

			if result:

				update_status_info(editor, msg0059, u"yes")

	# Make the buffer editable once again.

	editor.text_view.set_editable(True)

	# End recording for undo operation.

	editor.text_buffer.end_user_action()

	show_cursor(editor)

	return


def unindent(editor):
	u"""
	Unindent a line or selection of lines

	@param editor: Reference to the editor instance
	@type editor: An editor object

	"""

	show_cursor(editor, "busy")

	# Begin recording for undo operation.

	editor.text_buffer.begin_user_action()

	# Make the buffer uneditable to prevent interruption and possible corruption
	# off iterators by the user.

	editor.text_view.set_editable(False)

	# We use this flag to fool and prevent the editor from switching to edit
	# mode when the user types into the buffer.

	editor.mode_edit_flag = True

	# Retrieve the cursor line.

	cursor_mark = editor.text_buffer.get_insert()
	cursor_iter = editor.text_buffer.get_iter_at_mark(cursor_mark)
	cursor_line = cursor_iter.get_line()

	# Get selection boundaries

	selection = editor.text_buffer.get_selection_bounds()

	# If there is no selection indent the line anyway.

	if not selection:

		# String to push to the statusbar

		message = msg0147 % (cursor_line + 1)
		result = shift(editor, cursor_line, direction="left")

		# Send feedback to the statusbar indicating the result of the operation.

		update_status_info(editor, message, u"yes")

		if not result:

			# Make the buffer editable once again.

			editor.text_view.set_editable(True)

			# End recording for undo operation.

			editor.text_buffer.end_user_action()

			# Send feedback to the statusbar indicating the result of the
			# operation.

			update_status_info(editor, msg0060, u"no")
			beep()
			show_cursor(editor)

			return

	else:

		# Get selection boundaries

		start_line = selection[0].get_line()
		end_line = selection[1].get_line()

		# If the selection is on the same line, indent the line

		if start_line == end_line:

			result = shift(editor, start_line, direction="left")

			if result:

				# String to push to the statusbar

				message = msg0147 % (start_line + 1)

				# Send feedback to the statusbar indicating the result of the
				# operation.

				update_status_info(editor, message, u"yes")

			else:

				# Send feedback to the statusbar indicating the result of the
				# operation.

				update_status_info(editor, msg0060, u"no")
				beep()

		else:

			# A count to keep track of unindented lines

			count = 0

			# Iterate through each line that is selected, and remove spaces at
			# the beginning of each line

			for lines in range(start_line, end_line+1):

				start_iter = editor.text_buffer.get_iter_at_line(lines)

				# If there are no characters on the line, move to the next line.

				if start_iter.ends_line():

					continue

				result = shift(editor, lines, direction="left")

				# Keep track of how many lines are unindented.

				if result:

					count += 1

			# Count is zero if no lines were indented. In that case, send
			# feedback to the statusbar indicating the result of the operation.

			if not count:

				update_status_info(editor, msg0148, u"no")
				beep()

			else:

				# Send feedback to the statusbar indicating the result of the
				# operation.

				update_status_info(editor, msg0061, u"yes")

	# Make the buffer editable once again.

	editor.text_view.set_editable(True)

	# End recording for undo operation.

	editor.text_buffer.end_user_action()

	show_cursor(editor)

	return


def remove_spaces(editor, where="end", line=None):
	u"""
	Remove spaces either at the beginning or end of line(s).

	The where parameter can takes one value, "begin" or "end"

	@param editor: Reference to an editor instance
	@type editor: An editor object

	@param where: Whether to removes spaces from the beginning or end of a line.
	@type where: A string object

	@param line: The line number to remove spaces from.
	@type line: An integer

	@return: True if the operation was successful and false otherwise.
	@rtype: A boolean object

	"""

	show_cursor(editor, "busy")

	# Get the iterators at the begining and end of the line.

	start_iter = editor.text_buffer.get_iter_at_line(line)


	if where == "begin":

		end_iter = start_iter.copy()

	else:

		end_iter = start_iter.copy()
		end_iter.forward_to_line_end()
		end_iter.backward_char()

	# Initialize the outcome of the remove spaces operation to false.

	result = False

	# Depending on the value of where, removes spaces either at the begining or
	# end of the line.

	while end_iter.get_char() == " " or end_iter.get_char() == "\t":

		result = True
		response()

		if where == "begin":

			end_iter.forward_char()

		else:

			end_iter.backward_char()

		if where == "end":

			if end_iter.equal(start_iter):

				new_iter = start_iter.copy()
				new_iter.forward_to_line_end()
				editor.text_buffer.delete(start_iter, new_iter)
				show_cursor(editor)

				return result

	# If result is true, delete spaces at the beginning of the line.

	if where == "begin" and result:

		editor.text_buffer.delete(start_iter, end_iter)

	# If result is false, delete spaces at the end of the line.

	if where == "end" and result:

		end_iter.forward_char()
		start_iter.forward_to_line_end()
		editor.text_buffer.delete(end_iter, start_iter)

	show_cursor(editor)

	return result


def remove_trailing_spaces(editor, flag=False):
	u"""
	Remove trailing spaces on line(s)

	@param editor: an editor instance
	@type editor: an editor object

	@param flag: whether or not feedback is sent to the statusbar
	@type flag: boolean
	"""

	show_cursor(editor, "busy")

	# Begin recording for undo operation.

	editor.text_buffer.begin_user_action()

	# Make the buffer uneditable to prevent interruption and possible corruption
	# off iterators by the user.

	editor.text_view.set_editable(False)

	# A flag used to indicate that the status icon should not be interrupted
	# when the buffer is modified. Used to prevent the edit icon mode from
	# being displayed.

	editor.mode_edit_flag = True

	# Retrieve the cursor line.

	cursor_mark = editor.text_buffer.get_insert()
	cursor_iter = editor.text_buffer.get_iter_at_mark(cursor_mark)
	cursor_line = cursor_iter.get_line()

	# Get selection bounds

	selection = editor.text_buffer.get_selection_bounds()

	# If there are no selection bounds, remove trailing spaces on the cursor
	# line. Otherwise, remove trailing spaces on all selected lines.

	if not selection:

		result = remove_spaces(editor, "end", cursor_line)

		# Feedback to user indicating the operation is successful.

		update_status_info(editor, msg0135, "yes")

	else:

		# Get the beginning and ending selection bounds.

		first_line = selection[0].get_line()
		last_line = selection[1].get_line()

		# If only one line is selected remove trailing spaces on that line

		if first_line == last_line:

			result = remove_spaces(editor, "end", cursor_line)

			# Feedback to user indicating the operation is successful.

			update_status_info(editor, msg0135, "yes")

		else:

			# Count keeps track of how many lines have been operated upon

			count = 0

			# Iterate through selected lines and removes trailing spaces, if
			# any, in each line.

			for lines in range(first_line, last_line+1):

				result = remove_spaces(editor, "end", lines)

				if result:

					count += 1

			# If trailing spaces were removed in any line, result is true.
			# Otherwise, result is false.

			if count:

				result = True

				# Feedback to user indicating the operation is successful.

				update_status_info(editor, msg0144, "yes")

	# If result is false, send feedback to the user via the statusbar indicating
	# the outcome of the operation.

	if not result:

		# Make the buffer editable once again.

		editor.text_view.set_editable(True)

		if not flag:

			# Make the buffer editable once again.

			editor.text_view.set_editable(True)

			# End recording for undo operation.

			editor.text_buffer.end_user_action()

			# Feedback to user when no trailing spaces are found.

			update_status_info(editor, msg0134, "no")
			beep()
			show_cursor(editor)

		return

	# Make the buffer editable once again.

	editor.text_view.set_editable(True)

	# End recording for undo operation.

	editor.text_buffer.end_user_action()

	show_cursor(editor)

	return


def remove_beginning_spaces(editor, flag=False):
	u"""
	Remove spaces at the beginning of a line

	@param editor: editor instance
	@type editor: editor object

	@param flag: whether or not feedback is sent to the statusbar
	@type flag: boolean

	"""

	show_cursor(editor, "busy")

	# Begin recording for undo operation.

	editor.text_buffer.begin_user_action()

	# Make the buffer uneditable to prevent interruption and possible corruption
	# off iterators by the user.

	editor.text_view.set_editable(False)

	# A flag used to indicate that the status icon should not be interrupted
	# when the buffer is modified. Used to prevent the edit icon mode from
	# being displayed.

	editor.mode_edit_flag = True

	# Retrieve the cursor line.

	cursor_mark = editor.text_buffer.get_insert()
	cursor_iter = editor.text_buffer.get_iter_at_mark(cursor_mark)
	cursor_line = cursor_iter.get_line()

	# Get selection bounds

	selection = editor.text_buffer.get_selection_bounds()

	# If there are no selection bounds, remove spaces on the cursor
	# line. Otherwise, remove spaces on all selected lines.

	if not selection:

		result = remove_spaces(editor, "begin", cursor_line)

		# Feedback to user indicating the operation is successful.

		update_status_info(editor, msg0142, "yes")

	else:

		# Get the beginning and ending selection bounds.

		first_line = selection[0].get_line()
		last_line = selection[1].get_line()

		# If only one line is selected remove spaces on that line

		if first_line == last_line:

			result = remove_spaces(editor, "begin", cursor_line)

			# Feedback to user indicating the operation is successful.

			update_status_info(editor, msg0142, "yes")

		else:

			# Count keeps track of how many lines have been operated upon

			count = 0

			# Iterate through selected lines and removes trailing spaces, if
			# any, in each line.

			for lines in range(first_line, last_line+1):

				result = remove_spaces(editor, "begin", lines)

				if result:

					count += 1

			# If trailing spaces were removed in any line, result is true.
			# Otherwise, result is false.

			if count:

				# Feedback to user indicating the operation is successful.

				update_status_info(editor, msg0143, "yes")
				result = True

	if not result:

		# Make the buffer editable once again.

		editor.text_view.set_editable(True)

		# Feedback to user when no spaces are found.

		if not flag:

			# Make the buffer editable once again.

			editor.text_view.set_editable(True)

			# End recording for undo operation.

			editor.text_buffer.end_user_action()

			# Feedback to user indicating the operation was not successful.

			update_status_info(editor, msg0141, "no")
			beep()
			show_cursor(editor)

			return

	# Make the buffer editable once again.

	editor.text_view.set_editable(True)

	# End recording for undo operation.

	editor.text_buffer.end_user_action()

	show_cursor(editor)

	return