# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""

This module contains the function necessary for line operations and
manipulations. Any operation that involves deleting, joining or adjusting lines
should be found here.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
@todo: Write a function to open a new line above or below the current line.
"""

from gtk import TEXT_SEARCH_VISIBLE_ONLY
from gtk.gdk import beep

from internationalization import msg0004, msg0005, msg0006, msg0136, msg0137
from internationalization import msg0138, msg0139, msg0140
from utility import update_status_info


def delete_line(editor):
	u"""
	Delete a line in the editor's buffer.

	@param editor: Reference to an editor instance.
	@type editor: An editor object.

	"""

	# Begin to record for possible undo actions

	editor.text_buffer.begin_user_action()

	# Get the line the cursor in on

	cursor_mark = editor.text_buffer.get_insert()
	cursor_iter = editor.text_buffer.get_iter_at_mark(cursor_mark)
	cursor_line = cursor_iter.get_line()

	# Get the iterators at the begining of the line

	start_iter = editor.text_buffer.get_iter_at_line(cursor_line)
	end_iter = start_iter.copy()

	# msg0004 is a string eligible for internationalization and localization.
	# It identifies the string "Deleted line integer." See the module
	# internationalization.py for more information.

	editor.mode_edit_flag = True
	message = msg0004 % (cursor_line + 1)

	# Feedback to the user via the statusbar

	update_status_info(editor, message, mode="yes")

	# If the character is a newline, we delete it and return

	if start_iter.get_char() == "\n" and not start_iter.is_end():

		end_iter.forward_char()
		editor.text_buffer.delete(start_iter, end_iter)
		editor.text_buffer.end_user_action()

		return

	# Get the iterator at the end of the line.

	end_iter.forward_to_line_end()
	end_iter.forward_char()

	# Delete the line

	if end_iter.is_end() and cursor_line > 0:

		start_iter = editor.text_buffer.get_iter_at_line(cursor_line-1)

		if start_iter.get_char() == "\n":

			end_iter = editor.text_buffer.get_end_iter()

		else:

			end_iter = start_iter.copy()
			end_iter.forward_to_line_end()
			start_iter = end_iter.copy()
			end_iter = editor.text_buffer.get_end_iter()

		editor.text_buffer.delete(start_iter, end_iter)
		editor.text_buffer.end_user_action()

		return

	editor.text_buffer.delete(start_iter, end_iter)
	editor.text_buffer.end_user_action()

	return


def join_line(editor):
	u"""
	Join lines in the editor's buffer

	@param editor: Reference to an editor instance.
	@type editor: An editor object.

	"""

	editor.mode_edit_flag = True

	try:

		# Begin to record for possible undo actions

		editor.text_buffer.begin_user_action()

		# Get the line the cursor in on

		cursor_mark = editor.text_buffer.get_insert()
		cursor_iter = editor.text_buffer.get_iter_at_mark(cursor_mark)
		cursor_line = cursor_iter.get_line()

		# Get the iterators at the begining of the line

		start_iter = editor.text_buffer.get_iter_at_line(cursor_line)
		end_iter = start_iter.copy()

		# Search for the newline character

		begin, end = end_iter.forward_search("\n", TEXT_SEARCH_VISIBLE_ONLY)

		# Create a mark at the beginning of the newline character

		mbegin = editor.text_buffer.create_mark(None, begin, left_gravity=False)

		# Delete the newline character to join lines.

		editor.text_buffer.delete(begin, end)
		insert_point = editor.text_buffer.get_iter_at_mark(mbegin)
		end_iter = insert_point.copy()

		# Search for space or tab characters if any.

		while end_iter.get_char() == " " or end_iter.get_char() == "\t":

			end_iter.forward_char()

		# Remove all space or tab characters.

		editor.text_buffer.delete(insert_point, end_iter)

		# Insert a space.

		insert_point = editor.text_buffer.get_iter_at_mark(mbegin)
		editor.text_buffer.insert(insert_point, " ")

		# Stop recording for possible undo actions

		editor.text_buffer.end_user_action()

		# msg0005 is a string eligible for internationalization and localization.
		# It identifies the string "Joined line integer to integer." See the
		# module internationalization.py for more information.

		message = msg0005 % (cursor_line + 2, cursor_line + 1)

		# Feedback to the user via the statusbar

		update_status_info(editor, message, "yes")

	except TypeError:

		# Stop recording for possible undo actions

		editor.text_buffer.end_user_action()

		# msg0006 is a string eligible for internationalization and localization.
		# It identifies the string "No more lines to join." See the
		# module internationalization.py for more information.

		update_status_info(editor, msg0006, "no")
		beep()

	return


def open_line_above(editor):
	u"""
	Add a newline character to the beginning of the current line

	@param editor: Reference to an editor instance.
	@type editor: An editor object.

	"""
	# A flag used to indicate that the status icon should not be interrupted
	# when the buffer is modified. Used to prevent the edit icon mode from
	# being displayed.

	editor.mode_edit_flag = True

	# Begin recording for undo operation.

	editor.text_buffer.begin_user_action()

	# Retrieve the cursor line.

	cursor_mark = editor.text_buffer.get_insert()
	cursor_iter = editor.text_buffer.get_iter_at_mark(cursor_mark)
	cursor_line = cursor_iter.get_line()

	# Get the iterators at the begining and end of the line.

	start_iter = editor.text_buffer.get_iter_at_line(cursor_line)

	# Insert a new line and move the cursor to the new line.

	editor.text_buffer.insert(start_iter, "\n")
	start_iter = editor.text_buffer.get_iter_at_line(cursor_line)
	editor.text_buffer.place_cursor(start_iter)

	# End recording for undo operation.

	editor.text_buffer.end_user_action()

	# Feedback to user indicating which line has been freed.

	update_status_info(editor, msg0136 % (cursor_line + 1), "yes")

	return


def open_line_below(editor):
	u"""
	Add a newline character to the end of the current line

	@param editor: an editor instance
	@type editor: an editor object

	"""

	# A flag used to indicate that the status icon should not be interrupted
	# when the buffer is modified. Used to prevent the edit icon mode from
	# being displayed.

	editor.mode_edit_flag = True

	# Begin recording for undo operation.

	editor.text_buffer.begin_user_action()

	# Retrieve the cursor line.

	cursor_mark = editor.text_buffer.get_insert()
	cursor_iter = editor.text_buffer.get_iter_at_mark(cursor_mark)
	cursor_line = cursor_iter.get_line()

	# Get the iterators at the begining of the line.

	start_iter = editor.text_buffer.get_iter_at_line(cursor_line)



	if start_iter.ends_line():

		# Get iterator at the beginning of next line

		next_iter = editor.text_buffer.get_iter_at_line(cursor_line+1)

		# Insert a newline character

		editor.text_buffer.insert(next_iter, "\n")

		# Move the cursor to the correct line

		start_iter = editor.text_buffer.get_iter_at_line(cursor_line+1)
		editor.text_buffer.place_cursor(start_iter)

		# End recording for undo operation.

		editor.text_buffer.end_user_action()

		# Feedback to user indicating which line has been freed.

		update_status_info(editor, msg0136 % (cursor_line + 2), "yes")

	else:

		start_iter.forward_to_line_end()

		# Insert a new line and move the cursor to the newline

		editor.text_buffer.insert(start_iter, "\n")
		start_iter = editor.text_buffer.get_iter_at_line(cursor_line+1)
		editor.text_buffer.place_cursor(start_iter)

		# End recording for undo operation.

		editor.text_buffer.end_user_action()

		# Feedback to user indicating which line has been freed.

		update_status_info(editor, msg0136 % (cursor_line + 2), "yes")

	return


def delete_cursor_to_line_end(editor):
	u"""
	Delete all text from the cursor to the end on the line.

	@param editor: Reference to an editor instance.
	@type editor: An editor object.

	"""

	# A flag used to indicate that the status icon should not be interrupted
	# when the buffer is modified. Used to prevent the edit icon mode from
	# being displayed.

	editor.mode_edit_flag = True

	# Begin recording for undo operation.

	editor.text_buffer.begin_user_action()

	# Retrieve the cursor line.

	cursor_mark = editor.text_buffer.get_insert()
	cursor_iter = editor.text_buffer.get_iter_at_mark(cursor_mark)

	# If the cursor is already at the end of the line, return

	if cursor_iter.ends_line():

		editor.text_buffer.end_user_action()
		update_status_info(editor, msg0138, "no")
		beep()

		return

	end_iter = cursor_iter.copy()
	end_iter.forward_to_line_end()
	editor.text_buffer.delete(cursor_iter, end_iter)

	# End recording for undo operation.

	editor.text_buffer.end_user_action()

	# Feedback to user indicating which line has been freed.

	update_status_info(editor, msg0137, "yes")

	return


def delete_cursor_to_line_begin(editor):
	u"""
	Delete all text from the cursor to the beginning of the line.

	@param editor: Reference to an editor instance.
	@type editor: An editor object.

	"""

	# A flag used to indicate that the status icon should not be interrupted
	# when the buffer is modified. Used to prevent the edit icon mode from
	# being displayed.

	editor.mode_edit_flag = True

	# Begin recording for undo operation.

	editor.text_buffer.begin_user_action()

	# Retrieve the cursor line.

	cursor_mark = editor.text_buffer.get_insert()
	cursor_iter = editor.text_buffer.get_iter_at_mark(cursor_mark)
	cursor_line = cursor_iter.get_line()

	# Get the iterators at the begining and end of the line.

	start_iter = editor.text_buffer.get_iter_at_line(cursor_line)

	# If the cursor is already at the beginning of the line, return.

	if cursor_iter.starts_line():

		editor.text_buffer.end_user_action()
		update_status_info(editor, msg0140, "no")
		beep()

		return

	editor.text_buffer.delete(start_iter, cursor_iter)

	# End recording for undo operation.

	editor.text_buffer.end_user_action()

	# Feedback to user indicating which line has been freed.

	update_status_info(editor, msg0139, "yes")

	return
