# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""
Displays the editor's current mode in the status area.

The editor modes serve primarily as feedback services for the user. Scribes
strives to avoid showing dialog messages to convey information to the user.
Thus, icons representing useful information appear at the leftmost corner of the
status area. The icons get updated depending on what the user is doing, or what
mode the editor is in.

@comment: This feature is largely experimental. We need to brainstorm on a large
	list of possible user actions and modes that can be represented with icons
	that can be conveyed to the user.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gobject import timeout_add
from gtk import Frame, STOCK_SPELL_CHECK, Image, STOCK_OPEN, STOCK_EDIT
from gtk import STOCK_SAVE, STOCK_SAVE_AS, STOCK_NEW, STOCK_PRINT, STOCK_UNDO
from gtk import STOCK_REDO, STOCK_JUMP_TO, STOCK_FIND, STOCK_FIND_AND_REPLACE
from gtk import STOCK_PREFERENCES, STOCK_DIALOG_WARNING
from gtk import STOCK_DIALOG_INFO, STOCK_DIALOG_AUTHENTICATION
from gtk import STOCK_DIALOG_ERROR, ICON_SIZE_MENU, STOCK_PASTE, STOCK_CUT
from gtk import STOCK_COPY, SHADOW_OUT, ProgressBar, STOCK_INDENT
from gtk import STOCK_UNINDENT, STOCK_HELP, STOCK_SELECT_FONT
from gtk import STOCK_YES, STOCK_NO, STOCK_JUSTIFY_CENTER, STOCK_JUSTIFY_RIGHT
from gtk import STOCK_JUSTIFY_FILL

from timeout import calculate_progress
from internationalization import msg0062

def update_editor_mode(editor, icon="", progress=False):
	u"""
	Display an icon in the status area representing the current mode the editor
	is in.

	Current modes supported

		- open/load mode: Editor is loading a file or the open dialog is
			activated
		- save mode: Editor is saving the current file
		- saveas mode: The save as dialog is activated
		- print mode: Printing is occuring or the print dialog activated
		- undo mode: An action has just been undone
		- redo mode: An undone action has just been redone
		- goto mode: The goto bar is active.
		- search mode: The find bar is active.
		- replace mode: The replace bar is active.
		- preferences mode: The preferences dialog is active
		- help mode: The help manual was activated
		- readonly mode: The documented can only be viewed but not edited
		- warning mode: A minor problem may have occured
		- error mode: A fatal error has occured
		- yes mode: An operation succeeded
		- no mode: An operation failed

	@param editor: Reference to an editor instance
	@type editor: An editor object

	@param icon: A stock icon that represents a mode or operation.
	@type icon: A string object

	@param progress: Whether or not to activate the progress bar
	@type progress: A boolean object

	@return: A mode indicating an editor operation
	@rtype: A gtk.Frame object containing a Stock icon

	@note: This modes behavior is still in its experimental phase. It is very
		   likely that the whole system and behavior will be redesigned.

	"""

	modes = {
		"open": STOCK_OPEN,
		"edit": STOCK_EDIT,
		"save": STOCK_SAVE,
		"saveas": STOCK_SAVE_AS,
		"new": STOCK_NEW,
		"print": STOCK_PRINT,
		"undo": STOCK_UNDO,
		"redo": STOCK_REDO,
		"goto": STOCK_JUMP_TO,
		"find": STOCK_FIND,
		"replace": STOCK_FIND_AND_REPLACE,
		"preferences": STOCK_PREFERENCES,
		"help": STOCK_HELP,
		"warning": STOCK_DIALOG_WARNING,
		"info": STOCK_DIALOG_INFO,
		"readonly": STOCK_DIALOG_AUTHENTICATION,
		"error": STOCK_DIALOG_ERROR,
		"copy" : STOCK_COPY,
		"cut": STOCK_CUT,
		"paste": STOCK_PASTE,
		"spell": STOCK_SPELL_CHECK,
		"indent": STOCK_INDENT,
		"unindent": STOCK_UNINDENT,
		"help": STOCK_HELP,
		"font": STOCK_SELECT_FONT,
		"yes": STOCK_YES,
		"no": STOCK_NO,
		"tab": STOCK_JUSTIFY_CENTER,
		"text": STOCK_JUSTIFY_RIGHT,
		"margin": STOCK_JUSTIFY_FILL,
		"" : None
	}

	if progress:

		# Create the progress bar and set the progress flag

		editor.progress_flag = True
		progress = ProgressBar()
		progress.set_text(msg0062)
		progress.show()
		editor.timer = timeout_add(500, calculate_progress, editor, progress)

		return progress

	else:

		# Create a frame to hold stock icons

		mode = Frame()
		mode.set_no_show_all(True)
		mode.set_shadow_type(SHADOW_OUT)

		# Create image from stock icon

		image = Image()
		mode.add(image)
		image.set_from_stock(modes[icon], ICON_SIZE_MENU)
		image.show()

		return mode