# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""
Manage cursor position and window states.

This module contains code that keeps track of the editor's cursor position and
window state. The only function exported in this module is window_position. It
is called before a file is opened or closed.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from os import path, makedirs
from shelve import open

def window_position(editor, mode="get", filename=""):
	u"""
	Get or set the editor's window position.

	Before a file is closed, this function gets the cursor position and window
	state of the editor object.	The data retrieved is then stored in the
	editor's database, ~/.gnome2/scribesinfo.gdb.

	Before a file is opened, this function checks the database to see if
	there is an entry for it in the database. If there is one, the function sets
	the editor object's cursor position and window state based on the
	information retrieved from the database.

	@param editor: the editor.
	@type editor: an editor object
	@param mode: "get" or "set". "get" is used to retrieve the
		window state and cursor position. The retrieved information is stored in
		a database. "set" is used to set the window state and cursor position
		based on information provided by the database.
	@type mode: a string
	@param filename: the name of the text file
	@type filename: a string

	@return: a boolean flag, and/or tuple containing the file's window state and
		cursor position.
	@rtype: boolean, and/or tuple
	"""

	# If the database folder does not exist, create one

	if not path.exists(editor.SCRIBESINFODIR):

		makedirs(editor.SCRIBESINFODIR)

	# Open the editor's metadata database. The database stores information about
	# a file's window position (i.e window cordinates), state (i.e maximize
	# or unmaximize) and the position of the cursor in file before it was is
	# closed.

	metadata = open(editor.SCRIBESINFODIR + "/scribesinfo.gdb", "c")

	# Get the cursor position in the editor's buffer and window state.

	if mode == u"get":

		if editor.fullscreen_flag:

			editor.maximize_flag = True

		rect = (editor.x, editor.y, editor.width, editor.height)
		metadata[str(filename)] = (editor.cursor, editor.maximize_flag, rect)
		metadata.close()

		return

	# Set the cursor position and window state of an editor object.

	if mode == "set":

		# If the file has a key in the database continue processing. Otherwise
		# close the database and exit.

		if metadata.has_key(str(filename)):

			# The flag indicates that information about the file exists in the
			# database.

			editor.position_flag = True
			editor.pos = metadata[str(filename)]
			metadata.close()

			if editor.pos[1]:

				editor.maximize()

				return

			if editor.pos[2]:

				editor.move(editor.pos[2][0], editor.pos[2][1])
				editor.resize(editor.pos[2][2], editor.pos[2][3])

				return

		else:

			# The flag indicates that information about the file does not exist
			# in the database.

			editor.position_flag = False
			metadata.close()

		return