# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""
This module defines user customization options for Scribes
"""

from gtk import Dialog, WIN_POS_CENTER_ON_PARENT, STOCK_CLOSE, RESPONSE_CLOSE
from gtk import Label, Alignment, HBox, FontButton, SpinButton, CheckButton
from gtk import UPDATE_ALWAYS, keysyms
from gtk.gdk import WINDOW_TYPE_HINT_DIALOG, GRAVITY_CENTER

from scribes_gconf import get_gconf_client
from tooltips import create_tooltips, font_button_tip, tab_spin_button_tip
from tooltips import tw_check_button_tip, margin_check_button_tip
from tooltips import margin_spin_button_tip, spell_check_button_tip
from internationalization import msg0063, msg0064, msg0065, msg0066, msg0067
from internationalization import msg0068, msg0069, msg0070, msg0071, msg0072
from internationalization import msg0073, msg0132


class Preferences(object):
	u"""
	Creates a dialog containing editor options which users can tweak.
	"""

	def __init__(self, editor):
		u"""
		Initialize and show the preferences dialog window

		@param self: Reference to the preference dialog window
		@type self: a gtk.Dialog object

		@param editor: Reference to the editor instance
		@type editor: An editor object

		"""
		# Initialize gconf, the gnome configuration system

		client = get_gconf_client()

		self.ed = editor

		# Display user feedback icon in status area.

		self.ed.add_mode("preferences")

		tips = create_tooltips()

		# Create preferences dialog

		dialog = Dialog()
		dialog.connect(u"key_press_event", self.key_press_event_cb, dialog)

		# Set window properties

		dialog.set_title(msg0132)
		dialog.set_role(u"preferences_dialog")
		dialog.activate_default()
		dialog.set_transient_for(self.ed)
		dialog.set_type_hint(WINDOW_TYPE_HINT_DIALOG)
		dialog.set_modal(True)
		dialog.set_size_request(450, 450)
		dialog.set_skip_taskbar_hint(True)
		dialog.set_icon_name(u"gnome-settings")
		dialog.set_position(WIN_POS_CENTER_ON_PARENT)
		dialog.set_resizable(False)
		dialog.set_has_separator(False)
		dialog.set_gravity(GRAVITY_CENTER)

		# Set dialog properties

		dialog.add_button(STOCK_CLOSE, RESPONSE_CLOSE)


################################################################################
#
#							Font Config
#
################################################################################


		gconf_font_value = client.get_string("/apps/scribes/font")

		# Add a Font Label

		label = Label(msg0063)
		label.set_use_markup(True)
		label.set_alignment(0.025, 0.5)
		dialog.vbox.pack_start(label, False, False, 5)

		# Add the Font Selection label and widget and place them within an
		# alignment

		alignment = Alignment()
		font_container = HBox()
		alignment.add(font_container)
		alignment.set(0.5, 0.0, 0.7, 0.0)
		dialog.vbox.pack_start(alignment, True, True, 5)

		label = Label(msg0064)
		label.set_use_underline(True)
		font_container.pack_start(label, False, True, 5)

		self.font_button = FontButton(fontname=gconf_font_value)
		font_container.pack_start(self.font_button, True, True)
		self.font_button.connect("font-set", self.font_button_cb)
		tips.set_tip(self.font_button, font_button_tip)
		label.set_mnemonic_widget(self.font_button)
		client.notify_add("/apps/scribes/font", self.font_changed_cb,
						  self.font_button)


################################################################################
#
#							Tab Stops
#
################################################################################


		tab_value = client.get_int("/apps/scribes/tab")

		# Add a Tab Label

		string = msg0065
		label = Label(string)
		label.set_use_markup(True)
		label.set_alignment(0.025, 0.5)
		dialog.vbox.pack_start(label, False, False, 5)

		# Add a label and a tab control spin button within an alignment.

		alignment = Alignment()
		tab_container = HBox()
		alignment.add(tab_container)
		alignment.set(0.5, 0.0, 0.8, 0.0)

		dialog.vbox.pack_start(alignment, True, True, 5)

		string_one = msg0066
		label_one = Label(string_one)
		label_one.set_use_underline(True)

		tab_container.pack_start(label_one, False, True, 5)
		tab_spin_button = SpinButton()
		self.tab_spin_button = tab_spin_button
		tab_spin_button.connect("value-changed", self.tab_spin_button_cb)
		tips.set_tip(tab_spin_button, tab_spin_button_tip)
		label_one.set_mnemonic_widget(tab_spin_button)

		# Set Entry properties

		tab_spin_button.set_max_length(3)
		tab_spin_button.set_width_chars(3)

		# Set Spin Button properties

		tab_spin_button.set_digits(0)
		tab_spin_button.set_increments(1, 5)
		tab_spin_button.set_range(1, 24)
		tab_spin_button.set_update_policy(UPDATE_ALWAYS)
		tab_spin_button.set_numeric(True)
		tab_spin_button.set_snap_to_ticks(True)
		tab_container.pack_start(tab_spin_button, False, True)
		tab_spin_button.set_value(tab_value)
		client.notify_add("/apps/scribes/tab", self.tab_changed_cb,
						  self.tab_spin_button)


################################################################################
#
#						Text Wrapping
#
################################################################################


		text_wrapping_value = client.get_bool("/apps/scribes/text_wrapping")

		# Add a Text wrapping Label

		string = msg0067
		label = Label(string)
		label.set_use_markup(True)
		label.set_alignment(0.025, 0.5)
		dialog.vbox.pack_start(label, False, False, 5)

		# Add a check button within an alignment

		alignment = Alignment()
		string = msg0068
		text_wrapping_check_button = CheckButton(label=string,
												 use_underline=True)
		self.text_wrapping_check_button = text_wrapping_check_button
		text_wrapping_check_button.connect("toggled", self.wrapping_cb)
		tips.set_tip(text_wrapping_check_button, tw_check_button_tip)
		text_wrapping_check_button.set_active(text_wrapping_value)
		client.notify_add("/apps/scribes/text_wrapping",
						  self.wrapping_changed_cb,
						  self.text_wrapping_check_button)

		alignment.add(text_wrapping_check_button)
		alignment.set(0.5, 0.0, 0.8, 0.0)

		dialog.vbox.pack_start(alignment, True, True, 5)


################################################################################
#
#						Right Margin

################################################################################


		margin_line_value = client.get_bool("/apps/scribes/margin")
		margin_pos_value = client.get_int("/apps/scribes/margin_position")

		# Add a Line Margin Label

		string = msg0069
		label = Label(string)
		label.set_use_markup(True)
		label.set_alignment(0.025, 0.5)
		dialog.vbox.pack_start(label, False, False, 5)

		# Add a check button within an alignment

		alignment = Alignment()
		string = msg0070
		margin_check_button = CheckButton(label=string,
												 use_underline=True)
		self.margin_check_button = margin_check_button
		margin_check_button.connect("toggled", self.margin_line_cb)
		tips.set_tip(margin_check_button, margin_check_button_tip)

		margin_check_button.set_active(margin_line_value)
		client.notify_add("/apps/scribes/margin",
						  self.margin_line_changed_cb,
						  self.margin_check_button)
		alignment.add(margin_check_button)
		alignment.set(0.5, 0.0, 0.8, 0.0)

		dialog.vbox.pack_start(alignment, True, True, 5)

		# Add a label and a tab control spin button within an alignment.

		alignment = Alignment()
		right_margin_container = HBox()
		alignment.add(right_margin_container)
		alignment.set(0.5, 0.0, 0.7, 0.0)

		dialog.vbox.pack_start(alignment, True, True, 5)

		string_one = msg0071
		label_one = Label(string_one)
		label_one.set_use_underline(True)
		right_margin_container.pack_start(label_one, False, True, 5)
		margin_spin_button = SpinButton()
		label_one.set_mnemonic_widget(margin_spin_button)
		self.margin_spin_button = margin_spin_button
		margin_spin_button.connect("value-changed", self.margin_pos_cb)
		tips.set_tip(margin_spin_button, margin_spin_button_tip)

		# Set Entry properties

		margin_spin_button.set_max_length(3)
		margin_spin_button.set_width_chars(3)

		# Set Spin Button properties

		margin_spin_button.set_digits(0)
		margin_spin_button.set_increments(1, 5)
		margin_spin_button.set_range(1, 160)
		margin_spin_button.set_value(margin_pos_value)
		margin_spin_button.set_update_policy(UPDATE_ALWAYS)
		margin_spin_button.set_numeric(True)
		margin_spin_button.set_snap_to_ticks(True)
		right_margin_container.pack_start(margin_spin_button, False, True)
		client.notify_add("/apps/scribes/margin_position",
						  self.margin_pos_changed_cb, self.margin_spin_button)

		if not margin_line_value:

			margin_spin_button.set_sensitive(False)

		else:

			margin_spin_button.set_sensitive(True)


################################################################################
#
#						Spell Checking
#
################################################################################


		spell_value = client.get_bool("/apps/scribes/spell_check")

		# Add a Spell Checking Label

		string = msg0072
		label = Label(string)
		label.set_use_markup(True)
		label.set_alignment(0.025, 0.5)
		dialog.vbox.pack_start(label, False, False, 5)

		# Add a check button within an alignment

		alignment = Alignment()
		string = msg0073
		spell_check_button = CheckButton(label=string,
												 use_underline=True)
		self.spell_check_button = spell_check_button
		spell_check_button.connect("toggled", self.spelling_cb)
		spell_check_button.set_active(spell_value)
		client.notify_add("/apps/scribes/text_wrapping",
						  self.spelling_changed_cb,
						  self.spell_check_button)
		tips.set_tip(spell_check_button, spell_check_button_tip)

		alignment.add(spell_check_button)
		alignment.set(0.5, 0.0, 0.8, 0.0)

		dialog.vbox.pack_start(alignment, True, True, 5)

		dialog.show_all()
		response = dialog.run()

		if response:

			dialog.hide_all()
			dialog.destroy()
			self.ed.reset_mode()
			del self


################################################################################
#
#						Key Press Event Callback
#
################################################################################


	def key_press_event_cb(self, entry, key_press, dialog):
		u"""Callback to the dialog's "key-press-event"

		The "key-press-event" signal is emitted when the user presses a key on
		the	keyboard.

		key_press -- Escape
		self -- Preference dialog

		"""

		if key_press.keyval == keysyms.Escape:

			dialog.response(RESPONSE_CLOSE)


################################################################################
#
#					Font Button Callback
#
################################################################################


	def font_button_cb(self, widget):
		u"""
		Handles callback when the font button in the preferences' dialog window
		is clicke.

		@param self: Reference to the preference dialog window
		@type self: a gtk.Dialog object

		@param widget: Reference to the font button in the preferences dialog
		@type widget: a gtk.Button object

		"""

		client = get_gconf_client()
		font = widget.get_font_name()
		client.set_string("/apps/scribes/font", font)

		return

	def font_changed_cb(self, client, cnxn_id, entry, font_button):
		u"""
		Handles callback to alert gconf when the font type and/or size changes

		@param self: Reference to the preference dialog window
		@type self: a gtk.Dialog object

		@param client: Reference to the gconf client
		@type client: a gconf client object

		@param cnxn_id: connection ID from gconf_client_notify_add().
		@type cnxn_id: An integer object

		@param entry: ?
		@type entry: ?

		@param font_button: Reference to the font button in the preferences
							dialog
		@type font_button: a gtk.Button object

		"""

		font_button.set_font_name(entry.value.to_string())

		return

################################################################################
#
#					Tab Button Callback
#
################################################################################


	def tab_spin_button_cb(self, widget):
		u"""
		Handles callback when the values of the tab spin button in the
		preferences dialog window is changed.

		@param self: Reference to the preference dialog window
		@type self: a gtk.Dialog object

		@param widget: Reference to the spin button in the preferences dialog
		@type widget: a gtk.SpinButton object

		"""

		client = get_gconf_client()
		new_value = widget.get_value()
		client.set_int("/apps/scribes/tab", int(new_value))


	def tab_changed_cb(self, client, cnxn_id, entry, tab):
		u"""
		Handles callback to alert gconf when tab with size changes

		@param self: Reference to the preference dialog window
		@type self: a gtk.Dialog object

		@param client: Reference to the gconf client
		@type client: a gconf client object

		@param cnxn_id: connection ID from gconf_client_notify_add().
		@type cnxn_id: An integer object

		@param entry: ?
		@type entry: ?

		@param font_button: Reference to the spin button in the preferences
							dialog
		@type font_button: a gtk.SpinButton object

		"""

		tab.set_value(int(entry.value.to_string()))

		return


################################################################################
#
#					Text Wrapping Callback
#
################################################################################

	def wrapping_cb(self, widget):
		u"""
		Handles callback when the text wrapping check button in the preferences
		dialog window is toggled on or off.

		@param self: Reference to the preference dialog window
		@type self: a gtk.Dialog object

		@param widget: Reference to the text wrapping check button in the
					   preferences dialog
		@type widget: a gtk.CheckButton object

		"""

		client = get_gconf_client()
		new_value = widget.get_active()
		client.set_bool("/apps/scribes/text_wrapping", new_value)

		return


	def wrapping_changed_cb(self, client, cnxn_id, entry, check_button):
		u"""
		Handles callback to alert gconf when text wrapping is toggled on or off.

		@param self: Reference to the preference dialog window
		@type self: a gtk.Dialog object

		@param client: Reference to the gconf client
		@type client: a gconf client object

		@param cnxn_id: connection ID from gconf_client_notify_add().
		@type cnxn_id: An integer object

		@param entry: ?
		@type entry: ?

		@param font_button: Reference to the text wrapping check button in the
							preferences dialog
		@type font_button: a gtk.CheckButton object

		"""

		return


################################################################################
#
#					Right Margin Callback
#
################################################################################


	def margin_line_cb(self, widget):
		u"""
		Handles callback when the right margin line check button in the
		preferences dialog window is toggled on or off.

		@param self: Reference to the preference dialog window
		@type self: a gtk.Dialog object

		@param widget: Reference to the right margin line check button in the
					   preferences dialog
		@type widget: a gtk.CheckButton object

		"""

		client = get_gconf_client()
		new_value = widget.get_active()

		try:

			if not new_value:

				self.margin_spin_button.set_sensitive(False)

			else:

				self.margin_spin_button.set_sensitive(True)

		except:

			pass

		client.set_bool("/apps/scribes/margin", new_value)

		return


	def margin_line_changed_cb(self, client, cnxn_id, entry,
							   margin_check_button):
		u"""
		Handles callback to alert gconf when right margin line activated or
		deactivated.

		@param self: Reference to the preference dialog window
		@type self: a gtk.Dialog object

		@param client: Reference to the gconf client
		@type client: a gconf client object

		@param cnxn_id: connection ID from gconf_client_notify_add().
		@type cnxn_id: An integer object

		@param entry: ?
		@type entry: ?

		@param font_button: Reference to the margin line position check button
							in the preferences dialog
		@type font_button: a gtk.CheckButton object

		"""
		return


	def margin_pos_cb(self, widget):
		u"""
		Handles callback when the right margin position spin button in the
		preferences dialog window is changed.

		@param self: Reference to the preference dialog window
		@type self: a gtk.Dialog object

		@param widget: Reference to the right margin position button in the
					   preferences dialog
		@type widget: a gtk.SpinButton object

		"""

		client = get_gconf_client()
		new_value = widget.get_value()
		client.set_int("/apps/scribes/margin_position", int(new_value))

		return


	def margin_pos_changed_cb(self, client, cnxn_id, entry, margin_button):
		u"""
		Handles callback to alert gconf when right margin line position changes.

		@param self: Reference to the preference dialog window
		@type self: a gtk.Dialog object

		@param client: Reference to the gconf client
		@type client: a gconf client object

		@param cnxn_id: connection ID from gconf_client_notify_add().
		@type cnxn_id: An integer object

		@param entry: ?
		@type entry: ?

		@param font_button: Reference to the margin position spin button in the
							preferences dialog
		@type font_button: a gtk.CheckButton object

		"""

		margin_button.set_value(int(entry.value.to_string()))

		return


################################################################################
#
#					Spell Checking Callback
#
################################################################################

	def spelling_cb(self, widget):
		u"""
		Handles callback when the spell checking check button in the
		preferences dialog window is toggled on or off.

		@param self: Reference to the preference dialog window
		@type self: a gtk.Dialog object

		@param widget: Reference to the right margin line check button in the
					   preferences dialog
		@type widget: a gtk.CheckButton object

		"""

		client = get_gconf_client()
		new_value = widget.get_active()

		if client.get_bool("/apps/scribes/spell_check") != new_value:

			client.set_bool("/apps/scribes/spell_check", new_value)

		return


	def spelling_changed_cb(self, client, cnxn_id, entry, spell_check_button):
		u"""
		Handles callback to alert gconf when spell checking is activated or
		deactivated.

		@param self: Reference to the preference dialog window
		@type self: a gtk.Dialog object

		@param client: Reference to the gconf client
		@type client: a gconf client object

		@param cnxn_id: connection ID from gconf_client_notify_add().
		@type cnxn_id: An integer object

		@param entry: ?
		@type entry: ?

		@param font_button: Reference to the spell checking check button in the
							preferences dialog
		@type font_button: a gtk.CheckButton object

		"""

		return