# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""
This module defines the functions responsible for creating and displaying the
save file dialog.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""

from os import path, environ

from gobject import timeout_add
from gtk.gdk import WINDOW_TYPE_HINT_DIALOG, GRAVITY_CENTER
from gtk import FileChooserDialog, WIN_POS_CENTER_ON_PARENT, STOCK_CANCEL
from gtk import RESPONSE_CANCEL, STOCK_SAVE, RESPONSE_OK
from gtk import FILE_CHOOSER_ACTION_SAVE, Expander, Alignment

from gnomevfs import format_uri_for_display
from timeout import syntax_highlight_timeout
from internationalization import msg0034, msg0082, msg0025, msg0173
from encoding import create_encoding_box


def search_folder_path(editor):
	u"""
	Search for default folder path

	@return: default folder path to use
	@rtype: string

	"""

	if editor.filename != msg0025:

		file_path = path.expanduser(editor.filename)
		folder_path = path.dirname(file_path)

		return folder_path

	if path.exists(editor.DESKTOPDIR):

		return editor.DESKTOPDIR

	else:

		return editor.HOMEDIR


def process_file(editor, filename):
	u"""
	Prepare the file for saving by defining filename semantics

	@param editor: editor
	@type editor: editor object
	@param filename: name of the text file to save on the host's filesystem
	@type filename: string

	"""

	editor.file_uri = filename
	editor.filename =	format_uri_for_display(filename)

	if editor.filename.startswith(environ['HOME']):

		editor.prettyname = editor.filename.replace(environ['HOME'], "~")

	else:

		editor.prettyname = editor.filename

	editor.isNewFile = False
	editor.actiongroup.get_action("save_file").activate()
	timeout_add(1000, syntax_highlight_timeout, editor)

	return


def set_name(editor):
	u"""
	Set the default name of the file to be presented in the save file dialog

	@param editor: editor
	@type editor: editor object

	@return: name of the text file
	@rtype: string

	"""

	if editor.get_title().startswith(u"*"):

		return editor.get_title().replace(u"*", u"", 1)

	elif editor.get_title().endswith(msg0034):

		return editor.get_title().replace(msg0034, u"", 1)

	else:

		return editor.get_title()


def show_save_dialog(editor):

	u"""
	Show the save dialog.

	The save dialog allows users to save of a file with a new name.

	@param editor: editor
	@type editor: editor object

	"""

	# Display icon mode in status area

	editor.add_mode(u"saveas")

	# Create file selector dialog

	dialog = FileChooserDialog()

	# Set window properties

	dialog.set_title(msg0082)
	dialog.set_role(u"save_dialog")
	dialog.activate_default()
	dialog.set_transient_for(editor)
	dialog.set_type_hint(WINDOW_TYPE_HINT_DIALOG)
	dialog.set_modal(True)
	dialog.set_skip_taskbar_hint(True)
	dialog.set_icon_name(u"stock_save")
	dialog.set_position(WIN_POS_CENTER_ON_PARENT)
	dialog.set_gravity(GRAVITY_CENTER)

	# Set dialog properties

	dialog.add_button(STOCK_CANCEL, RESPONSE_CANCEL)
	dialog.add_button(STOCK_SAVE, RESPONSE_OK)
	dialog.set_default_response(RESPONSE_OK)

	# Set file chooser properties

	dialog.set_action(FILE_CHOOSER_ACTION_SAVE)
	dialog.set_current_folder(search_folder_path(editor))
	dialog.set_current_name(set_name(editor))

	# Create an Expander

	expander = Expander()
	dialog.vbox.pack_start(expander, False, False, 0)
	expander.set_label(msg0173)
	expander.set_use_underline(True)

	# Container that holds encoding selection combobox and label

	encoding_box = create_encoding_box(editor, "save")
	alignment = Alignment()
	alignment.add(encoding_box)
	alignment.set(0.3, 0.0, 0.7, 0.0)
	expander.add(alignment)

	encoding_box.show_all()

	dialog.show_all()

	response = dialog.run()

	if response == RESPONSE_OK:

		filename = dialog.get_uri()
		dialog.hide_all()
		dialog.destroy()
		process_file(editor, filename)
		editor.add_mode("save")

	else:

		dialog.hide_all()
		dialog.destroy()
		editor.reset_mode()

	return
