# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""
User configuration module for Scribes

Scribes uses GConf for managing user configurations. GConf consists of a daemon
and a database. The database contains the user configurations. The daemon
listens for changes made to the application's configuration and updates the
database instantly.

Scribes currently allows users to set 8 configuration options. Users can change
the editor's font, buffer tab width, text wrapping mode, right margin display
and position and spell checker. In addition, users can set or unset case
sensitivity and match type during searches.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from locale import getdefaultlocale
from gobject import GError

from gconf import client_get_default, CLIENT_PRELOAD_ONELEVEL
from pango import FontDescription
from gtk import WRAP_NONE, WRAP_WORD
from gtkspell import Spell


################################################################################
#
#				 		Gconf callbacks
#
################################################################################


def font_changed_cb(client, cnxn_id, entry, editor):
	u"""
	Handles callback when user makes changes to editor's font.

	The notify function is invoked when the value of a key changes.

	client -- gconf client
	cnxn_id -- connection id from notification daemon
	entry --
	editor -- editor object

	"""

	new_font = FontDescription(entry.value.to_string())
	editor.text_view.modify_font(new_font)
	editor.text_view.queue_draw()

	return

def tab_width_cb(client, cnxn_id, entry, editor):
	u"""Handles callback when user makes changes to editor's tab width

	The notify function is invoked when the value of a key changes.

	client -- gconf client
	cnxn_id -- connection id from notification daemon
	entry --
	editor -- editor object

	"""

	editor.text_view.set_tabs_width(int(entry.value.to_string()))
	editor.text_view.queue_draw()

	return


def text_wrapping_cb(client, cnxn_id, entry, editor):
	u"""Handles callback when user makes changes to editor's wrapping mode

	The notify function is invoked when the value of a key changes.

	client -- gconf client
	cnxn_id -- connection id from notification daemon
	entry --
	editor -- editor object

	"""

	if entry.value.get_bool():

		editor.text_view.set_wrap_mode(WRAP_WORD)

	else:

		editor.text_view.set_wrap_mode(WRAP_NONE)

	editor.text_view.queue_draw()

	return


def margin_position_cb(client, cnxn_id, entry, editor):
	u"""Handles callback when user makes changes to editor's position

	The notify function is invoked when the value of a key changes.

	client -- gconf client
	cnxn_id -- connection id from notification daemon
	entry --
	editor -- editor object

	"""

	editor.text_view.set_margin(int(entry.value.to_string()))
	editor.text_view.queue_draw()

	return


def show_margin_cb(client, cnxn_id, entry, editor):
	u"""Handles callback when user makes changes to editor's margin display

	The notify function is invoked when the value of a key changes.

	client -- gconf client
	cnxn_id -- connection id from notification daemon
	entry --
	editor -- editor object

	"""

	editor.text_view.set_show_margin(entry.value.get_bool())
	editor.text_view.queue_draw()

	return


def spell_check_cb(client, cnxn_id, entry, editor):
	u"""Handles callback when user makes changes to editor's spell checker

	The notify function is invoked when the value of a key changes.

	client -- gconf client
	cnxn_id -- connection id from notification daemon
	entry --
	editor -- editor object

	"""

	if entry.value.get_bool():

		try:

			editor.spell_checker = Spell(editor.text_view, getdefaultlocale()[0])

		except GError:

			pass

		except AttributeError:

			pass

	else:

		try:

			editor.spell_checker.detach()

		except AttributeError:

			pass

	editor.text_view.queue_draw()

	return


def match_case_cb(client, cnxn_id, entry, editor):
	u"""Handles callback when user makes changes to editor's search parameter

	The notify function is invoked when the value of a key changes.

	client -- gconf client
	cnxn_id -- connection id from notification daemon
	entry --
	editor -- editor object

	"""

	pass


def match_word_cb(client, cnxn_id, entry, editor):
	u"""Handles callback when user makes changes to editor's search parameter

	The notify function is invoked when the value of a key changes.

	client -- gconf client
	cnxn_id -- connection id from notification daemon
	entry --
	editor -- editor object

	"""

	pass


################################################################################
#
#					Get User Configuration Values (GCONF)
#
################################################################################


def scribes_init_gconf(editor):
	u"""
	Initialize the editor's configuration sources

	@param editor: editor
	@type editor: editor object

	"""

	# Initialize gconf, the gnome configuration system

	client = client_get_default()
	client.add_dir("/apps/scribes", CLIENT_PRELOAD_ONELEVEL)

	# Get GConf Values

	gconf_value_font = client.get_string("/apps/scribes/font")
	editor.font = FontDescription(gconf_value_font)
	editor.wrap_mode_bool = client.get_bool("/apps/scribes/text_wrapping")
	editor.tab_width = client.get_int("/apps/scribes/tab")
	editor.margin_position = client.get_int("/apps/scribes/margin_position")
	editor.show_margin = client.get_bool("/apps/scribes/margin")
	editor.spell_check = client.get_bool("/apps/scribes/spell_check")
	editor.match_case_flag = client.get_bool("/apps/scribes/match_case")
	editor.match_word_flag = client.get_bool("/apps/scribes/match_word")

	# GConf notification deamons

	client.notify_add("/apps/scribes/font", font_changed_cb, editor)
	client.notify_add("/apps/scribes/tab", tab_width_cb, editor)
	client.notify_add("/apps/scribes/text_wrapping", text_wrapping_cb, editor)
	client.notify_add("/apps/scribes/margin", show_margin_cb, editor)
	client.notify_add("/apps/scribes/margin_position", margin_position_cb, editor)
	client.notify_add("/apps/scribes/spell_check", spell_check_cb, editor)
	client.notify_add("/apps/scribes/match_case", match_case_cb, editor)
	client.notify_add("/apps/scribes/match_word", match_word_cb, editor)

	return


def get_gconf_client():
	u"""
	Get a gconf client

	@return:	A gconf client
	@rtype:		A gconf client object

	"""

	client = client_get_default()
	client.add_dir("/apps/scribes", CLIENT_PRELOAD_ONELEVEL)

	return client