# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""
Provide syntax higlighting for the editor's buffer.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com

"""

from gtk.gdk import color_parse
from gtksourceview import SourceLanguagesManager, SourceTagStyle
from gnomevfs import get_mime_type


################################################################################
#
#							Syntax Highlight
#
################################################################################


def syntax_highlight(editor, filename):
	u"""
	Highlight keywords in source code for various programming languages.

	@param editor: editor
	@type editor: an editor object
	@param filename: name of text file to be highlighted
	@type filename: string

	@todo: Provide comprehensive highlight for most common language
	@attention: Highlight code is heavily Python biased
	"""

	manager = SourceLanguagesManager()

	# Retrieve the mime type of the file. Used to detect which syntax highlight
	# to use.

	mime_type = get_mime_type(filename)

	if mime_type:

		# Syntax color and style for integers

		decimal_tag_style = SourceTagStyle()
		decimal_tag_style.foreground = color_parse("#8B6914")

		# Syntax color and sytle for floating point numbers.

		floating_point_number_tag_style = SourceTagStyle()
		floating_point_number_tag_style.foreground = color_parse("#FFA500")

		# Syntax color and style for functions.

		functions_tag_style = SourceTagStyle()
		functions_tag_style.foreground = color_parse("#A52A2A")
		functions_tag_style.bold = True

		# Syntax color and style for hexidecimal numbers

		hex_tag_style = SourceTagStyle()
		hex_tag_style.foreground = color_parse("#7474B3")

		# Syntax color and style for strings

		string_tag_style = SourceTagStyle()
		string_tag_style.foreground = color_parse("#0E657F")

		# Syntax color and style for multiline comments

		multiline_tag_style = SourceTagStyle()
		multiline_tag_style.foreground = color_parse("#8B6914")

		# Syntax color and style for single line comments

		line_comment_tag_style = SourceTagStyle()
		line_comment_tag_style.foreground = color_parse("#676790")
		line_comment_tag_style.italic = True

		# Get programming language from mime type

		language = manager.get_language_from_mime_type(mime_type)

		# Apply syntax tags

		if language:

			language.set_tag_style("Decimal", decimal_tag_style)
			language.set_tag_style("Floating@32@Point@32@Number",
								   floating_point_number_tag_style)
			language.set_tag_style("Functions", functions_tag_style)
			language.set_tag_style("Hex@32@Number", hex_tag_style)
			language.set_tag_style("Octal@32@Number", hex_tag_style)
			language.set_tag_style("Line@32@Comment",
										line_comment_tag_style)
			language.set_tag_style("String@32@2",
										string_tag_style)
			language.set_tag_style("String", string_tag_style)
			language.set_tag_style("Multiline@32@String@32@2",
									string_tag_style)
			language.set_tag_style("Multiline@32@String",
										multiline_tag_style)
			language.set_tag_style("Keywords", functions_tag_style)

			# Activate syntax highlight

			editor.text_buffer.set_highlight(True)
			editor.text_buffer.set_language(language)

		else:

			editor.text_buffer.set_highlight(False)

	else:

		editor.text_buffer.set_highlight(False)

	return