# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""
Textbuffer functions for Scribes

"""

from os import path

from gobject import timeout_add, source_remove
from gtksourceview import SourceBuffer

from timeout import save_file_timeout
from autocomplete import validate_cursor_string
from bookmark import update_bookmark_list, check_for_line_bookmark


################################################################################
#
#						Text Buffer Callbacks
#
################################################################################


def buffer_changed_cb(textbuffer, editor):
	u"""
	Handles callback when text is inserted into the buffer

	The "changed" signal is emitted when text is inserted in textbuffer.

	@param textbuffer:	editor buffer object
	@type textbuffer: 	A gtksourceview.SourceBuffer object
	@param editor:	A reference to the editor instance
	@type editor:		An editor object

	"""

	# A flag indicating the editor's buffer has been tainted.

	editor.dirty = True

	# This flag shows the file has already been saved and doesn't need
	# further saving. Used by the automatic saving function of the
	# editor. When the flag is true, the file is saved. Otherwise, the
	# file is unsaved.

	editor.save_file_flag = False

	# If the file is not a new file and text has been inserted into the
	# editor's display area, then prepend an asterik to the name of the
	# file in the editor window's title.

	if not editor.isNewFile:

		window_title = u"*" + path.basename(path.expanduser(editor.filename))
		editor.set_title(window_title)

	return


def modified_cb(textbuffer, editor):
	u"""
	Handles callback when the modification flag of the buffer has changed

	The "modified-changed" signal is emitted when the modification flag is
	changed.

	@param textbuffer:	editor buffer object
	@type textbuffer: 	A gtksourceview.SourceBuffer object
	@param editor:	A reference to the editor instance
	@type editor:		An editor object

	"""

	try:

		# Remove all save timers to avoid conflict

		source_remove(editor.save_timer)

	except AttributeError:

		pass

	# Check the saved state of the file. If the file has already been saved
	# exit this function.

	if editor.save_file_flag:

		return

	if not editor.mode_edit_flag:

		editor.add_mode("edit")
		editor.mode_edit_flag = True

	# If the file is not a new file, call the save function after 21 secs.

	if not editor.isNewFile:

		editor.save_timer = timeout_add(21000, save_file_timeout, editor)

	return


def can_undo_cb(textbuffer, boolean, editor):
	u"""Handles callback to determine whether undo events can occur.

	The "can-undo" signal is emitted when there is a change in the buffer's
	ability to undo an operation.

	@param textbuffer:	editor buffer object
	@type textbuffer: 	A gtksourceview.SourceBuffer object
	@param boolean:		Whether or not an undo procedure is possible
	@type boolean:		A boolean object
	@param editor:		A reference to the editor instance
	@type editor:			An editor object

	"""

	if boolean:

		editor.actiongroup.get_action(u"undo_edits").set_sensitive(True)

	else:

		editor.actiongroup.get_action(u"undo_edits").set_sensitive(False)

	return


def can_redo_cb(textbuffer, boolean, editor):
	u"""
	Handles callback to determine whether redo events can occur.

	The "can-redo" signal is emitted when there is a change in the buffer's
	ability to redo an operation.

	@param textbuffer:	editor buffer object
	@type textbuffer: 	A gtksourceview.SourceBuffer object
	@param boolean:		Whether or not an undo procedure is possible
	@type boolean:		A boolean object
	@param editor:		A reference to the editor instance
	@type editor:			An editor object

	"""

	if boolean:

		editor.actiongroup.get_action(u"redo_edits").set_sensitive(True)

	else:

		editor.actiongroup.get_action(u"redo_edits").set_sensitive(False)

	return


def insert_text_cb(textbuffer, iterator, text, length, editor):
	u"""
	Handles callback when text is inserted into the editor's buffer.

	The "insert-text" signal is emitted when text of the size specified by
	length is inserted into textbuffer at the location specified by iter.

	@param textbuffer:	The textbuffer that received the signal.
	@type textbuffer: A gtk.TextBuffer.

	@param iterator: Iterator of inserted text.
	@type iterator:	A gtk.TextIter.

	@param text: The text inserted in textbuffer.
	@type text: A string.

	@param length: The length of the text inserted in textbuffer.
	@type length: An integer.

	@param editor: A reference to the editor.
	@type editor: An editor object.

	"""

	# Check for the possibility of word completion

	validate_cursor_string(editor)

	return


def mark_set_cb(textbuffer, iterator, textmark, editor):
	u"""
	Handles callback when markers are deleted from the text editor's buffer.

	@param textbuffer: The text editor's buffer.
	@type textbuffer: A gtksourceview.SourceBuffer

	@param textmark: Marker to that has been set.
	@type textmark: A gtksourceview.Marker.

	@param editor: Reference to the text editor instance.
	@type editor: An editor object.

	"""

	try:

		if editor.bookmark_flag:

			if textmark in editor.bookmark_list:

				bookmarks = check_for_line_bookmark(editor)

				if bookmarks:

					editor.actiongroup.get_action("remove_bookmark").activate()

	except AttributeError:

		pass

	return


def mark_deleted_cb(textbuffer, textmark, editor):
	u"""
	Handles callback when markers are deleted from the text editor's buffer.

	@param textbuffer: The text editor's buffer.
	@type textbuffer: A gtksourceview.SourceBuffer

	@param textmark: Marker to be removed or deleted.
	@type textmark: A gtksourceview.Marker.

	@param editor: Reference to the text editor instance.
	@type editor: An editor object.

	"""

	try:

		if editor.bookmark_flag:

			if textmark in editor.bookmark_list:

				update_bookmark_list(editor, "remove", textmark)

	except AttributeError:

		pass

	return


################################################################################
#
#							Text Buffer
#
################################################################################


def create_text_buffer(editor):
	u"""
	Create a text buffer for the editor

	@param editor:	A reference to the editor instance
	@type editor:		An editor object

	@return:		text buffer object
	@rtype:			A gtksourceview.SourceBuffer object

	"""

	editor = editor
	text_buffer = SourceBuffer()
	editor.text_buffer = text_buffer
	editor.tb_chg_id = text_buffer.connect(u"changed", buffer_changed_cb, editor)
	editor.tb_mod_id = text_buffer.connect(u"modified-changed", modified_cb, editor)
	editor.tb_ins_id = text_buffer.connect_after(u"insert-text", insert_text_cb, editor)
	text_buffer.connect(u"can-undo", can_undo_cb, editor)
	text_buffer.connect(u"can-redo", can_redo_cb, editor)
	text_buffer.connect(u"mark-deleted", mark_deleted_cb, editor)
	text_buffer.connect(u"mark-set", mark_set_cb, editor)
	text_buffer.set_max_undo_levels(0)
	text_buffer.set_text(u"")
	text_buffer.set_modified(False)

	return text_buffer