# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""
This module contains timeout functions. They are callback functions called after
a certain time has elasped.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gc import collect

from gobject import source_remove
from gtk import TEXT_WINDOW_TEXT
from gtk.gdk import Cursor, Pixmap, Color

from syntax import syntax_highlight
from internationalization import msg0025, msg0044, msg0045, msg0095, msg0096


###############################################################################
#
#						Timeout Functions
#
################################################################################


def load_string_timeout(editor):
	u"""
	Show the URI of the open text file in the editor's status bar.

	@param editor: 	editor
	@type editor: 	an editor object

	@return : 		False
	@rtype: 		boolean
	"""

	editor = editor

	# If name is "Unsaved Document" don't show any message in the status bar.

	if editor.filename == msg0025:

		editor.status_one.pop(editor.status_one_id)
		editor.status_one_id = editor.status_one.get_context_id("")
		editor.status_one.push(editor.status_one_id, "")

		return False

	# If the file is in readonly mode, show the readonly icon in the statusbar
	# and display a message saying the file in open in readonly mode. Otherwise
	# show the absolute path to the file and show the correct icon mode.

	if editor.readonly:

		editor.add_mode("readonly")
		string = msg0044 % editor.prettyname

	else:

		editor.reset_mode()
		string = msg0045 % editor.prettyname

	if editor.message_id:

		editor.status_one.pop(editor.status_one_id)

	editor.status_one_id = editor.status_one.get_context_id(string)
	editor.status_one.push(editor.status_one_id, string)

	return False

def save_file_timeout(editor):
	u"""
	Save a modified document after 21 secs of modification.

	@param editor: 	editor
	@type editor: 	an editor object

	@return : 		False
	@rtype: 		boolean
	"""

	editor = editor

	# Save the file if it has not already been saved.

	if not editor.save_file_flag:

		editor.actiongroup.get_action(u"save_file").activate()

	return False

def syntax_highlight_timeout(editor):
	u"""
	Highlight buffer after a text file has been saved.

	@param editor: 	editor
	@type editor: 	an editor object

	@return : 		False
	@rtype: 		boolean
	"""

	editor = editor
	syntax_highlight(editor, editor.file_uri)

	return False


def drag_and_drop_timeout(editor):
	u"""
	Allow drag and drop 3 seconds after a file has been loaded.

	We do this to prevent the drag_data_received signal from being called twice.

	@param editor: 	editor
	@type editor: 	editor object
	"""

	editor.text_view.handler_unblock(editor.txtid)

	return False


def user_action_timeout(editor):
	u"""
	Handles a timeout callback to reset the statusbar's icon mode.

	The icon mode can either be reset to "new", "edit" or "readonly."

	@param editor:	Reference to the editor instance
	@type editor:		An editor object

	"""

	editor.reset_mode()

	try:

		# The flag indicates that status area is currently busy and should not
		# be updated until this operation is over

		editor.status_busy_flag = False

	except AttributeError:

		pass

	return False


def activate_progressbar_timeout(editor):
	u"""
	Show the progressbar widget in the statusbar area after 1 sec when a file
	is being loaded.

	@param editor:	Reference to the editor instance
	@type editor:		An editor object

	"""

	if editor.loading_flag:

		editor.add_mode("", progress=True)

	return False


def calculate_progress(editor, progress):
	u"""
	Calculate the progress bar fraction

	@param editor:		Reference to the editor instance
	@type editor:			An editor object
	@param progress:	Progress bar widget
	@type progress: 	GTK progressbar

	"""

	try:

		amount_loaded = float(editor.opt_count)/float(editor.file_size)
		amount_left = float(editor.opt_size)/float(editor.file_size)
		value = amount_loaded + amount_left
		string = msg0095 % (int(editor.opt_count / editor.file_size * 100))
		progress.set_text(string)

	except ZeroDivisionError:

		value = 0.0

	if not editor.progress_flag or value > 1.0:

		progress.set_text(msg0096)
		editor.add_mode("open")
		source_remove(editor.timer)
		source_remove(editor.gtimer)
		progress.hide()
		editor.add_mode("open")
		progress.destroy()

		return False

	if value >= 0.0 and value <= 1.0:

		progress.set_fraction(value)

	return True


def fullscreen_timeout(editor):
	u"""
	Handles callbact to hide the toolbar after a few seconds of pointer
	inactivity during fullscreen mode.

	@param editor:		Reference to the editor instance
	@type editor:			An editor object

	"""

	if not editor.fullscreen_flag:

		return False

	else:

		if editor.toolbar_flag:

			editor.toolbar_area.hide_all()
			editor.toolbar_flag = False

		return False


def quit_mainloop(editor):
	u"""
	Quit the application

	@return : False
	@rtype: boolean
	"""

	editor.actiongroup.get_action(u"close_file").activate()

	return False


def garbage(editor):
	u"""
	Handles a timeout callback to call the garbage collector every 30 minutes

	@return: Returns True so the timeout is called repeatedly
	@type: boolean

	"""

	# Run the garbage collector to free unused objects in memory. Do not run
	# the collector if the file is saving. Bad things happen.

	if not editor.saving_flag:

		collect()

	return True


def hide_cursor_timeout(editor):
	u"""
	Handles a timeout callback to hide the cursor after 5 seconds of cursor
	inactivity.

	@return: Returns True so the timeout is called repeatedly
	@type: boolean

	"""


	try:

		# Check to see if the cursor is busy. If it is don't hide the cursor.

		if editor.busy_cursor:

			return True

	except AttributeError:

		pass

	try:

		# Check to see if the editor.hidden_cursor has already been initialized.
		# If it hasn't, an exception is thrown and it is initialized to false.

		if editor.hidden_cursor:

			pass

	except AttributeError:

		editor.hidden_cursor = False

	try:

		# If the cursor is not hidden, hide it.

		if not editor.hidden_cursor:

			textview_win = editor.text_view.get_window(TEXT_WINDOW_TEXT)
			pixmap = Pixmap(None, 1, 1, 1)
			color = Color()
			textview_win.set_cursor(Cursor(pixmap, pixmap, color, color, 0, 0))
			editor.hidden_cursor = True

	except AttributeError:

		pass

	return True

