# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""
Common miscellaneous functions used by the editor

This module helper functions that perform activities such as finding Scribes
data and module files, pushing feedback to the editor's status bar and creating
customized buttons.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from sys import argv
from os import path

from gobject import timeout_add, source_remove
from gtk.gdk import beep
from gtk import HBox, Image, ICON_SIZE_BUTTON, Label, events_pending
from gtk import main_iteration

from timeout import load_string_timeout, user_action_timeout
from internationalization import msg0131


def find_app_data_dir():
	u"""
	Search for Scribes' data folder

	@return: path Scribes' data folder
	@rtype: string
	"""

	head, tail = path.split(path.split(path.abspath(argv[0]))[0])

	if tail ==  "bin":

		data_dir = path.join(head, "share")

		if path.exists(data_dir):

			return data_dir

	raise "FileNotFoundError", msg0131


def find_toolbar_folder():
	u"""
	Search for Scribes' toolbar folder

	@return: path to the toolbar folder
	@rtype: string
	"""

	head, tail = path.split(path.split(path.abspath(argv[0]))[0])

	if tail == 'bin':

		filepath = path.join(head, 'share', 'scribes')

		if path.isdir(filepath):

			return filepath

	raise "FileNotFoundError", msg0131


def create_button(stock_id, label):
	u"""
	Create a button with a stock icon and a custom label


	@param stock_id: a GTK stock icon identifier
	@type stock_id: string
	@param label: text to be displayed on the button
	@type label: GTK Label

	@return: a stock icon and label
	@rtype: HBox object
	"""

	hbox = HBox()
	image = Image()
	image.set_from_stock(stock_id, ICON_SIZE_BUTTON)
	label = Label(label)
	label.set_property(u"use-underline", True)
	hbox.pack_start(image)
	hbox.pack_start(label)

	return hbox


def show_status_info(editor, message, timeout=False, time=7000):
	u"""
	Push feedback information to the editor's statusbar

	@param editor: editor
	@type editor: an editor object
	@param message: feedback to display on the editor's statusbar
	@type message: string
	@param timeout: whether or not a timer function is called.
	@type timeout: boolean
	@param: the length of time before a timer function is activated
	@type: number
	"""

	if editor.message_id:

		editor.status_one.pop(editor.status_one_id)

	editor.status_one_id = editor.status_one.get_context_id(message)
	editor.status_one.push(editor.status_one_id, message)

	if timeout:

		editor.status_timer = timeout_add(time, load_string_timeout, editor)

		return


def response():
	u"""
	Allow scribes GUI to be responsive when performing tasks that can
	potentially	lock the GUI.

	@warning: 	Use this function sparringly and preferably within loops in
				callbacks. Never use this function in a timeout callback.
	@note:		There is a tradeoff between responsiveness and performance, so
				test to see that when using this function, you have not
				introduced performance hits in other areas of the editor.

	"""

	while events_pending():

		main_iteration(False)

	return


def update_status_info(editor, message="", mode="", sound=False):
	u"""
	Update the status bar with an icon mode and message.

	@param editor:	Reference to editor instance
	@type editor:		An editor object
	@param message:	Message to send to the statusbar
	@type message:	A string
	@param mode:	stock icon to display in the statusbar
	@type mode: 	A string
	@param sound: Whether or not to make the speaker beep.
	@type sound: A gtk.gdk.beep

	"""
	editor.add_mode(mode)

	try:

		# The flag indicates that status area is currently busy and should not
		# be updated until this operation is over

		editor.status_busy_flag = True

		# Avoid conflicts with other status messages that may need to use the
		# status area after a certain period of time.

		source_remove(editor.status_timer)

		# Avoid conflicts with other status icon messages that may need to use the
		# status area after a certain period of time.

		source_remove(editor.mode_timer)

	except AttributeError:

		pass

	# Feedback to user via the statusbar

	show_status_info(editor, message, True)

	# Feeback to user via icon in the statusbar. Yes means the user's action is
	# successful. No means it wasn't.

	editor.mode_timer = timeout_add(7000, user_action_timeout, editor)

	if sound:

		beep()

	return

